@@ -1,5 +1,3 @@
-# -*- coding: utf-8 -*-
-
 # Copyright 2013 Metacloud, Inc.
 # Copyright 2012 OpenStack Foundation
 #
@@ -27,7 +25,7 @@ from keystone.common import controller
 from keystone.common import dependency
 from keystone import config
 from keystone import exception
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 from keystone.openstack.common import log
 
 
@@ -140,8 +138,8 @@ class Tenant(controller.V2Controller):
                 user_ref = self.identity_api.get_user(user_id)
             except exception.UserNotFound:
                 # Log that user is missing and continue on.
-                message = _("User %(user_id)s in project %(project_id)s "
-                            "doesn't exist.")
+                message = ("User %(user_id)s in project %(project_id)s "
+                           "doesn't exist.")
                 LOG.debug(message,
                           {'user_id': user_id, 'project_id': tenant_id})
             else:
