@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # Copyright 2012 Canonical Ltd.
 #
@@ -17,9 +15,14 @@
 
 import uuid
 
+import six
+
+from keystone.common import authorization
 from keystone.common import controller
 from keystone.common import dependency
+from keystone.common import wsgi
 from keystone import exception
+from keystone.i18n import _
 
 
 INTERFACES = ['public', 'internal', 'admin']
@@ -27,20 +30,25 @@ INTERFACES = ['public', 'internal', 'admin']
 
 @dependency.requires('catalog_api')
 class Service(controller.V2Controller):
+
+    @controller.v2_deprecated
     def get_services(self, context):
         self.assert_admin(context)
         service_list = self.catalog_api.list_services()
         return {'OS-KSADM:services': service_list}
 
+    @controller.v2_deprecated
     def get_service(self, context, service_id):
         self.assert_admin(context)
         service_ref = self.catalog_api.get_service(service_id)
         return {'OS-KSADM:service': service_ref}
 
+    @controller.v2_deprecated
     def delete_service(self, context, service_id):
         self.assert_admin(context)
         self.catalog_api.delete_service(service_id)
 
+    @controller.v2_deprecated
     def create_service(self, context, OS_KSADM_service):
         self.assert_admin(context)
         service_id = uuid.uuid4().hex
@@ -53,6 +61,8 @@ class Service(controller.V2Controller):
 
 @dependency.requires('catalog_api')
 class Endpoint(controller.V2Controller):
+
+    @controller.v2_deprecated
     def get_endpoints(self, context):
         """Merge matching v3 endpoint refs into legacy refs."""
         self.assert_admin(context)
@@ -77,27 +87,32 @@ class Endpoint(controller.V2Controller):
             legacy_ep['%surl' % endpoint['interface']] = endpoint['url']
         return {'endpoints': legacy_endpoints.values()}
 
+    @controller.v2_deprecated
     def create_endpoint(self, context, endpoint):
         """Create three v3 endpoint refs based on a legacy ref."""
         self.assert_admin(context)
 
         # according to the v2 spec publicurl is mandatory
         self._require_attribute(endpoint, 'publicurl')
+        # service_id is necessary
+        self._require_attribute(endpoint, 'service_id')
 
         legacy_endpoint_ref = endpoint.copy()
 
         urls = {}
         for i in INTERFACES:
             # remove all urls so they aren't persisted them more than once
-            if endpoint.get('%surl' % i) is not None:
+            url = '%surl' % i
+            if endpoint.get(url):
                 # valid urls need to be persisted
-                urls[i] = endpoint.pop('%surl' % i)
-            elif '%surl' % i in endpoint:
-                # null urls can be discarded
-                endpoint.pop('%surl' % i)
+                urls[i] = endpoint.pop(url)
+            elif url in endpoint:
+                # null or empty urls can be discarded
+                endpoint.pop(url)
+                legacy_endpoint_ref.pop(url)
 
         legacy_endpoint_id = uuid.uuid4().hex
-        for interface, url in urls.iteritems():
+        for interface, url in six.iteritems(urls):
             endpoint_ref = endpoint.copy()
             endpoint_ref['id'] = uuid.uuid4().hex
             endpoint_ref['legacy_endpoint_id'] = legacy_endpoint_id
@@ -109,6 +124,7 @@ class Endpoint(controller.V2Controller):
         legacy_endpoint_ref['id'] = legacy_endpoint_id
         return {'endpoint': legacy_endpoint_ref}
 
+    @controller.v2_deprecated
     def delete_endpoint(self, context, endpoint_id):
         """Delete up to three v3 endpoint refs based on a legacy ref ID."""
         self.assert_admin(context)
@@ -123,6 +139,90 @@ class Endpoint(controller.V2Controller):
             raise exception.EndpointNotFound(endpoint_id=endpoint_id)
 
 
+@dependency.requires('catalog_api')
+class CatalogV3(controller.V3Controller):
+    collection_name = 'catalog'
+
+    @controller.protected()
+    def get_catalog(self, context):
+        # TODO(dolphm): this method of accessing the auth context is terrible,
+        # but context needs to be refactored to always have reasonable values.
+        env_context = context.get('environment', {})
+        auth_context = env_context.get(authorization.AUTH_CONTEXT_ENV, {})
+        user_id = auth_context.get('user_id')
+        project_id = auth_context.get('project_id')
+
+        if not user_id or not project_id:
+            raise exception.Forbidden(
+                _('A project-scoped token is required to produce a service '
+                  'catalog.'))
+
+        # The V3Controller base methods mostly assume that you're returning
+        # either a collection or a single element from a collection, neither of
+        # which apply to the catalog. Because this is a special case, this
+        # re-implements a tiny bit of work done by the base controller (such as
+        # self-referential link building) to avoid overriding or refactoring
+        # several private methods.
+        return {
+            'catalog': self.catalog_api.get_v3_catalog(user_id, project_id),
+            'links': {
+                'self': CatalogV3.base_url(context)}}
+
+
+@dependency.requires('catalog_api')
+class RegionV3(controller.V3Controller):
+    collection_name = 'regions'
+    member_name = 'region'
+
+    def create_region_with_id(self, context, region_id, region):
+        """Create a region with a user-specified ID.
+
+        This method is unprotected because it depends on ``self.create_region``
+        to enforce policy.
+        """
+        if 'id' in region and region_id != region['id']:
+            raise exception.ValidationError(
+                _('Conflicting region IDs specified: '
+                  '"%(url_id)s" != "%(ref_id)s"') % {
+                      'url_id': region_id,
+                      'ref_id': region['id']})
+        region['id'] = region_id
+        return self.create_region(context, region)
+
+    @controller.protected()
+    def create_region(self, context, region):
+        ref = self._normalize_dict(region)
+
+        if not ref.get('id'):
+            ref = self._assign_unique_id(ref)
+
+        ref = self.catalog_api.create_region(ref)
+        return wsgi.render_response(
+            RegionV3.wrap_member(context, ref),
+            status=(201, 'Created'))
+
+    @controller.protected()
+    def list_regions(self, context):
+        refs = self.catalog_api.list_regions()
+        return RegionV3.wrap_collection(context, refs)
+
+    @controller.protected()
+    def get_region(self, context, region_id):
+        ref = self.catalog_api.get_region(region_id)
+        return RegionV3.wrap_member(context, ref)
+
+    @controller.protected()
+    def update_region(self, context, region_id, region):
+        self._require_matching_id(region_id, region)
+
+        ref = self.catalog_api.update_region(region_id, region)
+        return RegionV3.wrap_member(context, ref)
+
+    @controller.protected()
+    def delete_region(self, context, region_id):
+        return self.catalog_api.delete_region(region_id)
+
+
 @dependency.requires('catalog_api')
 class ServiceV3(controller.V3Controller):
     collection_name = 'services'
@@ -132,8 +232,15 @@ class ServiceV3(controller.V3Controller):
         super(ServiceV3, self).__init__()
         self.get_member_from_driver = self.catalog_api.get_service
 
+    def _validate_service(self, service):
+        if 'enabled' in service and not isinstance(service['enabled'], bool):
+            msg = _('Enabled field must be a boolean')
+            raise exception.ValidationError(message=msg)
+
     @controller.protected()
     def create_service(self, context, service):
+        self._validate_service(service)
+
         ref = self._assign_unique_id(self._normalize_dict(service))
         self._require_attribute(ref, 'type')
 
@@ -142,8 +249,9 @@ class ServiceV3(controller.V3Controller):
 
     @controller.filterprotected('type')
     def list_services(self, context, filters):
-        refs = self.catalog_api.list_services()
-        return ServiceV3.wrap_collection(context, refs, filters)
+        hints = ServiceV3.build_driver_hints(context, filters)
+        refs = self.catalog_api.list_services(hints=hints)
+        return ServiceV3.wrap_collection(context, refs, hints=hints)
 
     @controller.protected()
     def get_service(self, context, service_id):
@@ -153,6 +261,7 @@ class ServiceV3(controller.V3Controller):
     @controller.protected()
     def update_service(self, context, service_id, service):
         self._require_matching_id(service_id, service)
+        self._validate_service(service)
 
         ref = self.catalog_api.update_service(service_id, service)
         return ServiceV3.wrap_member(context, ref)
@@ -182,11 +291,19 @@ class EndpointV3(controller.V3Controller):
         ref = cls.filter_endpoint(ref)
         return super(EndpointV3, cls).wrap_member(context, ref)
 
+    def _validate_endpoint(self, endpoint):
+        if 'enabled' in endpoint and not isinstance(endpoint['enabled'], bool):
+            msg = _('Enabled field must be a boolean')
+            raise exception.ValidationError(message=msg)
+
     @controller.protected()
     def create_endpoint(self, context, endpoint):
+        self._validate_endpoint(endpoint)
+
         ref = self._assign_unique_id(self._normalize_dict(endpoint))
         self._require_attribute(ref, 'service_id')
         self._require_attribute(ref, 'interface')
+        self._require_attribute(ref, 'url')
         self.catalog_api.get_service(ref['service_id'])
 
         ref = self.catalog_api.create_endpoint(ref['id'], ref)
@@ -194,8 +311,9 @@ class EndpointV3(controller.V3Controller):
 
     @controller.filterprotected('interface', 'service_id')
     def list_endpoints(self, context, filters):
-        refs = self.catalog_api.list_endpoints()
-        return EndpointV3.wrap_collection(context, refs, filters)
+        hints = EndpointV3.build_driver_hints(context, filters)
+        refs = self.catalog_api.list_endpoints(hints=hints)
+        return EndpointV3.wrap_collection(context, refs, hints=hints)
 
     @controller.protected()
     def get_endpoint(self, context, endpoint_id):
@@ -205,6 +323,7 @@ class EndpointV3(controller.V3Controller):
     @controller.protected()
     def update_endpoint(self, context, endpoint_id, endpoint):
         self._require_matching_id(endpoint_id, endpoint)
+        self._validate_endpoint(endpoint)
 
         if 'service_id' in endpoint:
             self.catalog_api.get_service(endpoint['service_id'])
