@@ -16,6 +16,8 @@
 #    License for the specific language governing permissions and limitations
 #    under the License.
 
+import errno
+import re
 import socket
 import ssl
 import sys
@@ -24,13 +26,34 @@ import eventlet
 import eventlet.wsgi
 import greenlet
 
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 from keystone.openstack.common import log
 
 
 LOG = log.getLogger(__name__)
 
 
+class EventletFilteringLogger(log.WritableLogger):
+    # NOTE(morganfainberg): This logger is designed to filter out specific
+    # Tracebacks to limit the amount of data that eventlet can log. In the
+    # case of broken sockets (EPIPE and ECONNRESET), we are seeing a huge
+    # volume of data being written to the logs due to ~14 lines+ per traceback.
+    # The traceback in these cases are, at best, useful for limited debugging
+    # cases.
+    def __init__(self, *args, **kwargs):
+        super(EventletFilteringLogger, self).__init__(*args, **kwargs)
+        self.regex = re.compile(r'errno (%d|%d)' %
+                                (errno.EPIPE, errno.ECONNRESET), re.IGNORECASE)
+
+    def write(self, msg):
+        m = self.regex.search(msg)
+        if m:
+            self.logger.log(log.logging.DEBUG, 'Error(%s) writing to socket.',
+                            m.group(1))
+        else:
+            self.logger.log(self.level, msg.rstrip())
+
+
 class Server(object):
     """Server class to manage multiple WSGI sockets and applications."""
 
@@ -46,9 +69,29 @@ class Server(object):
         self.cert_required = False
         self.keepalive = keepalive
         self.keepidle = keepidle
+        self.socket = None
 
     def start(self, key=None, backlog=128):
         """Run a WSGI server with the given application."""
+
+        if self.socket is None:
+            self.listen(key=key, backlog=backlog)
+
+        self.greenthread = self.pool.spawn(self._run,
+                                           self.application,
+                                           self.socket)
+
+    def listen(self, key=None, backlog=128):
+        """Create and start listening on socket.
+
+        Call before forking worker processes.
+
+        Raises Exception if this has already been called.
+        """
+
+        if self.socket is not None:
+            raise Exception(_('Server can only listen once.'))
+
         LOG.info(_('Starting %(arg0)s on %(host)s:%(port)s'),
                  {'arg0': sys.argv[0],
                   'host': self.host,
@@ -88,9 +131,7 @@ class Server(object):
                 _socket.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE,
                                    self.keepidle)
 
-        self.greenthread = self.pool.spawn(self._run,
-                                           self.application,
-                                           _socket)
+        self.socket = _socket
 
     def set_ssl(self, certfile, keyfile=None, ca_certs=None,
                 cert_required=True):
@@ -104,6 +145,9 @@ class Server(object):
         if self.greenthread is not None:
             self.greenthread.kill()
 
+    def stop(self):
+        self.kill()
+
     def wait(self):
         """Wait until all servers have completed running."""
         try:
@@ -118,7 +162,11 @@ class Server(object):
         logger = log.getLogger('eventlet.wsgi.server')
         try:
             eventlet.wsgi.server(socket, application, custom_pool=self.pool,
-                                 log=log.WritableLogger(logger), debug=False)
+                                 log=EventletFilteringLogger(logger),
+                                 debug=False)
+        except greenlet.GreenletExit:
+            # Wait until all servers have completed running
+            pass
         except Exception:
             LOG.exception(_('Server error'))
             raise
