@@ -18,6 +18,7 @@
 
 """Utility methods for working with WSGI servers."""
 
+from oslo import i18n
 import routes.middleware
 import six
 import webob.dec
@@ -27,8 +28,7 @@ from keystone.common import config
 from keystone.common import dependency
 from keystone.common import utils
 from keystone import exception
-from keystone.openstack.common import gettextutils
-from keystone.openstack.common.gettextutils import _
+from keystone.i18n import _
 from keystone.openstack.common import importutils
 from keystone.openstack.common import jsonutils
 from keystone.openstack.common import log
@@ -85,8 +85,8 @@ def validate_token_bind(context, token_ref):
             LOG.info(_("Kerberos bind authentication successful"))
 
         elif bind_mode == 'permissive':
-            LOG.debug(_("Ignoring unknown bind for permissive mode: "
-                        "{%(bind_type)s: %(identifier)s}"),
+            LOG.debug(("Ignoring unknown bind for permissive mode: "
+                       "{%(bind_type)s: %(identifier)s}"),
                       {'bind_type': bind_type, 'identifier': identifier})
         else:
             LOG.info(_("Couldn't verify unknown bind: "
@@ -103,7 +103,7 @@ def best_match_language(req):
     if not req.accept_language:
         return None
     return req.accept_language.best_match(
-        gettextutils.get_available_languages('keystone'))
+        i18n.get_available_languages('keystone'))
 
 
 class BaseApplication(object):
@@ -178,7 +178,7 @@ class Application(BaseApplication):
         arg_dict = req.environ['wsgiorg.routing_args'][1]
         action = arg_dict.pop('action')
         del arg_dict['controller']
-        LOG.debug(_('arg_dict: %s'), arg_dict)
+        LOG.debug('arg_dict: %s', arg_dict)
 
         # allow middleware up the stack to provide context, params and headers.
         context = req.environ.get(CONTEXT_ENV, {})
@@ -187,9 +187,9 @@ class Application(BaseApplication):
         context['path'] = req.environ['PATH_INFO']
         context['host_url'] = req.host_url
         params = req.environ.get(PARAMS_ENV, {})
-        #authentication and authorization attributes are set as environment
-        #values by the container and processed by the pipeline.  the complete
-        #set is not yet know.
+        # authentication and authorization attributes are set as environment
+        # values by the container and processed by the pipeline.  the complete
+        # set is not yet know.
         context['environment'] = req.environ
         req.environ = None
 
@@ -200,6 +200,11 @@ class Application(BaseApplication):
         # TODO(termie): do some basic normalization on methods
         method = getattr(self, action)
 
+        # NOTE(morganfainberg): use the request method to normalize the
+        # response code between GET and HEAD requests. The HTTP status should
+        # be the same.
+        req_method = req.environ['REQUEST_METHOD'].upper()
+
         # NOTE(vish): make sure we have no unicode keys for py2.6.
         params = self._normalize_dict(params)
 
@@ -236,7 +241,8 @@ class Application(BaseApplication):
             return result
 
         response_code = self._get_response_code(req)
-        return render_response(body=result, status=response_code)
+        return render_response(body=result, status=response_code,
+                               method=req_method)
 
     def _get_response_code(self, req):
         req_method = req.environ['REQUEST_METHOD']
@@ -281,22 +287,43 @@ class Application(BaseApplication):
             # Accept either is_admin or the admin role
             self.policy_api.enforce(creds, 'admin_required', {})
 
-    def _require_attribute(self, ref, attr):
-        """Ensures the reference contains the specified attribute."""
-        if ref.get(attr) is None or ref.get(attr) == '':
-            msg = _('%s field is required and cannot be empty') % attr
+    def _attribute_is_empty(self, ref, attribute):
+        """Returns true if the attribute in the given ref (which is a
+        dict) is empty or None.
+        """
+        return ref.get(attribute) is None or ref.get(attribute) == ''
+
+    def _require_attribute(self, ref, attribute):
+        """Ensures the reference contains the specified attribute.
+
+        Raise a ValidationError if the given attribute is not present
+        """
+        if self._attribute_is_empty(ref, attribute):
+            msg = _('%s field is required and cannot be empty') % attribute
+            raise exception.ValidationError(message=msg)
+
+    def _require_attributes(self, ref, attrs):
+        """Ensures the reference contains the specified attributes.
+
+        Raise a ValidationError if any of the given attributes is not present
+        """
+        missing_attrs = [attribute for attribute in attrs
+                         if self._attribute_is_empty(ref, attribute)]
+
+        if missing_attrs:
+            msg = _('%s field(s) cannot be empty') % ', '.join(missing_attrs)
             raise exception.ValidationError(message=msg)
 
     def _get_trust_id_for_request(self, context):
         """Get the trust_id for a call.
 
         Retrieve the trust_id from the token
-        Returns None if token is is not trust scoped
+        Returns None if token is not trust scoped
         """
         if ('token_id' not in context or
                 context.get('token_id') == CONF.admin_token):
-            LOG.debug(_('will not lookup trust as the request auth token is '
-                        'either absent or it is the system admin token'))
+            LOG.debug(('will not lookup trust as the request auth token is '
+                       'either absent or it is the system admin token'))
             return None
 
         try:
@@ -577,29 +604,55 @@ class ExtensionRouter(Router):
         return _factory
 
 
-def render_response(body=None, status=None, headers=None):
+def render_response(body=None, status=None, headers=None, method=None):
     """Forms a WSGI response."""
-    headers = headers or []
+    if headers is None:
+        headers = []
+    else:
+        headers = list(headers)
     headers.append(('Vary', 'X-Auth-Token'))
 
     if body is None:
         body = ''
         status = status or (204, 'No Content')
     else:
-        body = jsonutils.dumps(body, cls=utils.SmarterEncoder)
-        headers.append(('Content-Type', 'application/json'))
+        content_types = [v for h, v in headers if h == 'Content-Type']
+        if content_types:
+            content_type = content_types[0]
+        else:
+            content_type = None
+        if content_type is None or content_type == 'application/json':
+            body = jsonutils.dumps(body, cls=utils.SmarterEncoder)
+            if content_type is None:
+                headers.append(('Content-Type', 'application/json'))
         status = status or (200, 'OK')
 
-    return webob.Response(body=body,
+    resp = webob.Response(body=body,
                           status='%s %s' % status,
                           headerlist=headers)
 
+    if method == 'HEAD':
+        # NOTE(morganfainberg): HEAD requests should return the same status
+        # as a GET request and same headers (including content-type and
+        # content-length). The webob.Response object automatically changes
+        # content-length (and other headers) if the body is set to b''. Capture
+        # all headers and reset them on the response object after clearing the
+        # body. The body can only be set to a binary-type (not TextType or
+        # NoneType), so b'' is used here and should be compatible with
+        # both py2x and py3x.
+        stored_headers = resp.headers.copy()
+        resp.body = b''
+        for header, value in six.iteritems(stored_headers):
+            resp.headers[header] = value
+
+    return resp
+
 
 def render_exception(error, context=None, request=None, user_locale=None):
     """Forms a WSGI response based on the current error."""
 
     error_message = error.args[0]
-    message = gettextutils.translate(error_message, desired_locale=user_locale)
+    message = i18n.translate(error_message, desired_locale=user_locale)
     if message is error_message:
         # translate() didn't do anything because it wasn't a Message,
         # convert to a string.
