@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,8 +13,8 @@
 # under the License.
 
 from keystone.common import sql
-from keystone.common.sql import migration
 from keystone import exception
+from keystone.i18n import _
 
 
 class ProjectEndpoint(sql.ModelBase, sql.DictBase):
@@ -31,20 +29,15 @@ class ProjectEndpoint(sql.ModelBase, sql.DictBase):
                             nullable=False)
 
 
-class EndpointFilter(sql.Base):
-    # Internal interface to manage the database
+class EndpointFilter(object):
 
-    def db_sync(self, version=None):
-        migration.db_sync(version=version)
-
-    @sql.handle_conflicts(type='project_endpoint')
+    @sql.handle_conflicts(conflict_type='project_endpoint')
     def add_endpoint_to_project(self, endpoint_id, project_id):
-        session = self.get_session()
+        session = sql.get_session()
         with session.begin():
             endpoint_filter_ref = ProjectEndpoint(endpoint_id=endpoint_id,
                                                   project_id=project_id)
             session.add(endpoint_filter_ref)
-            session.flush()
 
     def _get_project_endpoint_ref(self, session, endpoint_id, project_id):
         endpoint_filter_ref = session.query(ProjectEndpoint).get(
@@ -57,27 +50,40 @@ class EndpointFilter(sql.Base):
         return endpoint_filter_ref
 
     def check_endpoint_in_project(self, endpoint_id, project_id):
-        session = self.get_session()
+        session = sql.get_session()
         self._get_project_endpoint_ref(session, endpoint_id, project_id)
 
     def remove_endpoint_from_project(self, endpoint_id, project_id):
-        session = self.get_session()
+        session = sql.get_session()
         endpoint_filter_ref = self._get_project_endpoint_ref(
             session, endpoint_id, project_id)
         with session.begin():
             session.delete(endpoint_filter_ref)
-            session.flush()
 
     def list_endpoints_for_project(self, project_id):
-        session = self.get_session()
+        session = sql.get_session()
         query = session.query(ProjectEndpoint)
         query = query.filter_by(project_id=project_id)
         endpoint_filter_refs = query.all()
         return endpoint_filter_refs
 
     def list_projects_for_endpoint(self, endpoint_id):
-        session = self.get_session()
+        session = sql.get_session()
         query = session.query(ProjectEndpoint)
         query = query.filter_by(endpoint_id=endpoint_id)
         endpoint_filter_refs = query.all()
         return endpoint_filter_refs
+
+    def delete_association_by_endpoint(self, endpoint_id):
+        session = sql.get_session()
+        with session.begin():
+            query = session.query(ProjectEndpoint)
+            query = query.filter_by(endpoint_id=endpoint_id)
+            query.delete(synchronize_session=False)
+
+    def delete_association_by_project(self, project_id):
+        session = sql.get_session()
+        with session.begin():
+            query = session.query(ProjectEndpoint)
+            query = query.filter_by(project_id=project_id)
+            query.delete(synchronize_session=False)
