@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -15,26 +13,44 @@
 # under the License.
 
 
+from keystone import assignment
 from keystone.catalog import controllers as catalog_controllers
 from keystone.common import controller
 from keystone.common import dependency
-from keystone.identity import controllers as identity_controllers
+from keystone import notifications
 
 
-@dependency.requires('catalog_api', 'identity_api', 'endpoint_filter_api')
+@dependency.requires('assignment_api', 'catalog_api', 'endpoint_filter_api')
 class EndpointFilterV3Controller(controller.V3Controller):
 
+    def __init__(self):
+        super(EndpointFilterV3Controller, self).__init__()
+        notifications.register_event_callback(
+            'deleted', 'project',
+            self._on_project_or_endpoint_delete)
+        notifications.register_event_callback(
+            'deleted', 'endpoint',
+            self._on_project_or_endpoint_delete)
+
+    def _on_project_or_endpoint_delete(self, service, resource_type, operation,
+                                       payload):
+        project_or_endpoint_id = payload['resource_info']
+        if resource_type == 'project':
+            self.endpoint_filter_api.delete_association_by_project(
+                project_or_endpoint_id)
+        else:
+            self.endpoint_filter_api.delete_association_by_endpoint(
+                project_or_endpoint_id)
+
     @controller.protected()
     def add_endpoint_to_project(self, context, project_id, endpoint_id):
         """Establishes an association between an endpoint and a project."""
         # NOTE(gyee): we just need to make sure endpoint and project exist
         # first. We don't really care whether if project is disabled.
-        # The relationship can still be establed even with a disabled project
-        # as there are no security implications.
+        # The relationship can still be established even with a disabled
+        # project as there are no security implications.
         self.catalog_api.get_endpoint(endpoint_id)
-        self.identity_api.get_project(project_id)
-        # NOTE(gyee): we may need to cleanup any existing project-endpoint
-        # associations here if either project or endpoint is not found.
+        self.assignment_api.get_project(project_id)
         self.endpoint_filter_api.add_endpoint_to_project(endpoint_id,
                                                          project_id)
 
@@ -42,16 +58,14 @@ class EndpointFilterV3Controller(controller.V3Controller):
     def check_endpoint_in_project(self, context, project_id, endpoint_id):
         """Verifies endpoint is currently associated with given project."""
         self.catalog_api.get_endpoint(endpoint_id)
-        self.identity_api.get_project(project_id)
-        # TODO(gyee): we may need to cleanup any existing project-endpoint
-        # associations here if either project or endpoint is not found.
+        self.assignment_api.get_project(project_id)
         self.endpoint_filter_api.check_endpoint_in_project(endpoint_id,
                                                            project_id)
 
     @controller.protected()
     def list_endpoints_for_project(self, context, project_id):
         """Lists all endpoints currently associated with a given project."""
-        self.identity_api.get_project(project_id)
+        self.assignment_api.get_project(project_id)
         refs = self.endpoint_filter_api.list_endpoints_for_project(project_id)
 
         endpoints = [self.catalog_api.get_endpoint(
@@ -71,7 +85,7 @@ class EndpointFilterV3Controller(controller.V3Controller):
         self.catalog_api.get_endpoint(endpoint_id)
         refs = self.endpoint_filter_api.list_projects_for_endpoint(endpoint_id)
 
-        projects = [self.identity_api.get_project(
+        projects = [self.assignment_api.get_project(
             ref.project_id) for ref in refs]
-        return identity_controllers.ProjectV3.wrap_collection(context,
-                                                              projects)
+        return assignment.controllers.ProjectV3.wrap_collection(context,
+                                                                projects)
