@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,16 +14,20 @@
 
 """Main entry point into the Credentials service."""
 
+import abc
+
+import six
+
 from keystone.common import dependency
 from keystone.common import manager
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.openstack.common import log
 
 
 CONF = config.CONF
 
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 
 
 @dependency.provider('credential_api')
@@ -41,25 +43,29 @@ class Manager(manager.Manager):
         super(Manager, self).__init__(CONF.credential.driver)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class Driver(object):
     # credential crud
 
+    @abc.abstractmethod
     def create_credential(self, credential_id, credential):
         """Creates a new credential.
 
         :raises: keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def list_credentials(self, **filters):
         """List all credentials in the system applying filters.
 
         :returns: a list of credential_refs or an empty list.
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def get_credential(self, credential_id):
         """Get a credential by ID.
 
@@ -67,8 +73,9 @@ class Driver(object):
         :raises: keystone.exception.CredentialNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def update_credential(self, credential_id, credential):
         """Updates an existing credential.
 
@@ -76,12 +83,38 @@ class Driver(object):
                  keystone.exception.Conflict
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def delete_credential(self, credential_id):
         """Deletes an existing credential.
 
         :raises: keystone.exception.CredentialNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
+
+    @abc.abstractmethod
+    def delete_credentials_for_project(self, project_id):
+        """Deletes all credentials for a project."""
+        self._delete_credentials(lambda cr: cr['project_id'] == project_id)
+
+    @abc.abstractmethod
+    def delete_credentials_for_user(self, user_id):
+        """Deletes all credentials for a user."""
+        self._delete_credentials(lambda cr: cr['user_id'] == user_id)
+
+    def _delete_credentials(self, match_fn):
+        """Do the actual credential deletion work (default implementation).
+
+        :param match_fn: function that takes a credential dict as the
+                         parameter and returns true or false if the
+                         identifier matches the credential dict.
+        """
+        for cr in self.list_credentials():
+            if match_fn(cr):
+                try:
+                    self.credential_api.delete_credential(cr['id'])
+                except exception.CredentialNotFound:
+                    LOG.debug('Deletion of credential is not required: %s',
+                              cr['id'])
