@@ -20,10 +20,8 @@ import routes
 from keystone import assignment
 from keystone import auth
 from keystone import catalog
-from keystone.common import cache
 from keystone.common import wsgi
 from keystone import config
-from keystone.contrib import endpoint_filter
 from keystone import controllers
 from keystone import credential
 from keystone import identity
@@ -38,32 +36,6 @@ CONF = config.CONF
 LOG = log.getLogger(__name__)
 
 
-def load_backends():
-
-    # Configure and build the cache
-    cache.configure_cache_region(cache.REGION)
-
-    # Ensure that the identity driver is created before the assignment manager.
-    # The default assignment driver is determined by the identity driver, so
-    # the identity driver must be available to the assignment manager.
-    _IDENTITY_API = identity.Manager()
-
-    DRIVERS = dict(
-        assignment_api=assignment.Manager(),
-        catalog_api=catalog.Manager(),
-        credential_api=credential.Manager(),
-        endpoint_filter_api=endpoint_filter.Manager(),
-        identity_api=_IDENTITY_API,
-        policy_api=policy.Manager(),
-        token_api=token.Manager(),
-        trust_api=trust.Manager(),
-        token_provider_api=token.provider.Manager())
-
-    auth.controllers.load_auth_methods()
-
-    return DRIVERS
-
-
 def fail_gracefully(f):
     """Logs exceptions and aborts."""
     @functools.wraps(f)
@@ -83,8 +55,6 @@ def fail_gracefully(f):
 @fail_gracefully
 def public_app_factory(global_conf, **local_conf):
     controllers.register_version('v2.0')
-    conf = global_conf.copy()
-    conf.update(local_conf)
     return wsgi.ComposingRouter(routes.Mapper(),
                                 [assignment.routers.Public(),
                                  token.routers.Router(),
@@ -94,8 +64,6 @@ def public_app_factory(global_conf, **local_conf):
 
 @fail_gracefully
 def admin_app_factory(global_conf, **local_conf):
-    conf = global_conf.copy()
-    conf.update(local_conf)
     return wsgi.ComposingRouter(routes.Mapper(),
                                 [identity.routers.Admin(),
                                  assignment.routers.Admin(),
@@ -106,16 +74,12 @@ def admin_app_factory(global_conf, **local_conf):
 
 @fail_gracefully
 def public_version_app_factory(global_conf, **local_conf):
-    conf = global_conf.copy()
-    conf.update(local_conf)
     return wsgi.ComposingRouter(routes.Mapper(),
                                 [routers.Versions('public')])
 
 
 @fail_gracefully
 def admin_version_app_factory(global_conf, **local_conf):
-    conf = global_conf.copy()
-    conf.update(local_conf)
     return wsgi.ComposingRouter(routes.Mapper(),
                                 [routers.Versions('admin')])
 
@@ -123,8 +87,6 @@ def admin_version_app_factory(global_conf, **local_conf):
 @fail_gracefully
 def v3_app_factory(global_conf, **local_conf):
     controllers.register_version('v3')
-    conf = global_conf.copy()
-    conf.update(local_conf)
     mapper = routes.Mapper()
     v3routers = []
     for module in [assignment, auth, catalog, credential, identity, policy]:
