@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2013 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -14,13 +12,11 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import os
+import copy
 import uuid
 
-from keystone.common.sql import migration
-from keystone import contrib
-from keystone.openstack.common import importutils
-from keystone import tests
+# NOTE(morganfainberg): import endpoint filter to populate the SQL model
+from keystone.contrib import endpoint_filter  # flake8: noqa
 from keystone.tests import test_v3
 
 
@@ -29,38 +25,25 @@ class TestExtensionCase(test_v3.RestfulTestCase):
     EXTENSION_NAME = 'endpoint_filter'
     EXTENSION_TO_ADD = 'endpoint_filter_extension'
 
-    def setup_database(self):
-        self.conf_files = super(TestExtensionCase, self).config_files()
-        self.conf_files.append(
-            tests.testsdir('test_associate_project_endpoint_extension.conf'))
-        super(TestExtensionCase, self).setup_database()
-        package_name = "%s.%s.migrate_repo" % (contrib.__name__,
-                                               self.EXTENSION_NAME)
-        package = importutils.import_module(package_name)
-        self.repo_path = os.path.abspath(
-            os.path.dirname(package.__file__))
-        migration.db_version_control(version=None, repo_path=self.repo_path)
-        migration.db_sync(version=None, repo_path=self.repo_path)
+    def config_overrides(self):
+        super(TestExtensionCase, self).config_overrides()
+        self.config_fixture.config(
+            group='catalog',
+            driver='keystone.contrib.endpoint_filter.backends.catalog_sql.'
+                   'EndpointFilterCatalog')
 
     def setUp(self):
         super(TestExtensionCase, self).setUp()
         self.default_request_url = (
             '/OS-EP-FILTER/projects/%(project_id)s'
             '/endpoints/%(endpoint_id)s' % {
-            'project_id': self.default_domain_project_id,
-            'endpoint_id': self.endpoint_id})
-
-    def tearDown(self):
-        super(TestExtensionCase, self).tearDown()
-        self.conf_files.pop()
+                'project_id': self.default_domain_project_id,
+                'endpoint_id': self.endpoint_id})
 
 
-class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
-    """Test OS-EP-FILTER endpoint to project associations extension."""
+class EndpointFilterCRUDTestCase(TestExtensionCase):
 
-    # endpoint-project associations crud tests
-    # PUT
-    def test_create_endpoint_project_assoc(self):
+    def test_create_endpoint_project_association(self):
         """PUT /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Valid endpoint and project id test case.
@@ -70,7 +53,7 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
                  body='',
                  expected_status=204)
 
-    def test_create_endpoint_project_assoc_noproj(self):
+    def test_create_endpoint_project_association_with_invalid_project(self):
         """PUT OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid project id test case.
@@ -78,12 +61,12 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         """
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': uuid.uuid4().hex,
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': uuid.uuid4().hex,
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=404)
 
-    def test_create_endpoint_project_assoc_noendp(self):
+    def test_create_endpoint_project_association_with_invalid_endpoint(self):
         """PUT /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid endpoint id test case.
@@ -91,12 +74,12 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         """
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': self.default_domain_project_id,
-                 'endpoint_id': uuid.uuid4().hex},
+                     'project_id': self.default_domain_project_id,
+                     'endpoint_id': uuid.uuid4().hex},
                  body='',
                  expected_status=404)
 
-    def test_create_endpoint_project_assoc_unexpected_body(self):
+    def test_create_endpoint_project_association_with_unexpected_body(self):
         """PUT /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Unexpected body in request. The body should be ignored.
@@ -106,8 +89,7 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
                  body={'project_id': self.default_domain_project_id},
                  expected_status=204)
 
-    # HEAD
-    def test_check_endpoint_project_assoc(self):
+    def test_check_endpoint_project_association(self):
         """HEAD /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Valid project and endpoint id test case.
@@ -118,11 +100,11 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
                  expected_status=204)
         self.head('/OS-EP-FILTER/projects/%(project_id)s'
                   '/endpoints/%(endpoint_id)s' % {
-                  'project_id': self.default_domain_project_id,
-                  'endpoint_id': self.endpoint_id},
+                      'project_id': self.default_domain_project_id,
+                      'endpoint_id': self.endpoint_id},
                   expected_status=204)
 
-    def test_check_endpoint_project_assoc_noproj(self):
+    def test_check_endpoint_project_association_with_invalid_project(self):
         """HEAD /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid project id test case.
@@ -131,12 +113,12 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         self.put(self.default_request_url)
         self.head('/OS-EP-FILTER/projects/%(project_id)s'
                   '/endpoints/%(endpoint_id)s' % {
-                  'project_id': uuid.uuid4().hex,
-                  'endpoint_id': self.endpoint_id},
+                      'project_id': uuid.uuid4().hex,
+                      'endpoint_id': self.endpoint_id},
                   body='',
                   expected_status=404)
 
-    def test_check_endpoint_project_assoc_noendp(self):
+    def test_check_endpoint_project_association_with_invalid_endpoint(self):
         """HEAD /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid endpoint id test case.
@@ -145,31 +127,53 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         self.put(self.default_request_url)
         self.head('/OS-EP-FILTER/projects/%(project_id)s'
                   '/endpoints/%(endpoint_id)s' % {
-                  'project_id': self.default_domain_project_id,
-                  'endpoint_id': uuid.uuid4().hex},
+                      'project_id': self.default_domain_project_id,
+                      'endpoint_id': uuid.uuid4().hex},
                   body='',
                   expected_status=404)
 
-    # GET
-    def test_get_endpoint_project_assoc(self):
-        """GET /OS-EP-FILTER/projects/{project_id}/endpoints success."""
-        self.put(self.default_request_url)
-        r = self.get('/OS-EP-FILTER/projects/%(project_id)s/endpoints' % {
-                     'project_id': self.default_domain_project_id})
-        self.assertValidEndpointListResponse(r, self.endpoint)
+    def test_list_endpoints_associated_with_valid_project(self):
+        """GET /OS-EP-FILTER/projects/{project_id}/endpoints
 
-    def test_get_endpoint_project_assoc_noproj(self):
-        """GET /OS-EP-FILTER/projects/{project_id}/endpoints no project."""
+        Valid project and endpoint id test case.
+
+        """
+        self.put(self.default_request_url)
+        resource_url = '/OS-EP-FILTER/projects/%(project_id)s/endpoints' % {
+                       'project_id': self.default_domain_project_id}
+        r = self.get(resource_url)
+        self.assertValidEndpointListResponse(r, self.endpoint,
+                                             resource_url=resource_url)
+
+    def test_list_endpoints_associated_with_invalid_project(self):
+        """GET /OS-EP-FILTER/projects/{project_id}/endpoints
+
+        Invalid project id test case.
+
+        """
         self.put(self.default_request_url)
         self.get('/OS-EP-FILTER/projects/%(project_id)s/endpoints' % {
                  'project_id': uuid.uuid4().hex},
                  body='',
                  expected_status=404)
 
-    def test_list_projects_for_endpoint_default(self):
-        """GET /OS-EP-FILTER/endpoints/{endpoint_id}/projects success
+    def test_list_projects_associated_with_endpoint(self):
+        """GET /OS-EP-FILTER/endpoints/{endpoint_id}/projects
 
-        Don't associate project and endpoint, then get empty list.
+        Valid endpoint-project association test case.
+
+        """
+        self.put(self.default_request_url)
+        resource_url = '/OS-EP-FILTER/endpoints/%(endpoint_id)s/projects' % {
+                       'endpoint_id': self.endpoint_id}
+        r = self.get(resource_url)
+        self.assertValidProjectListResponse(r, self.default_domain_project,
+                                            resource_url=resource_url)
+
+    def test_list_projects_with_no_endpoint_project_association(self):
+        """GET /OS-EP-FILTER/endpoints/{endpoint_id}/projects
+
+        Valid endpoint id but no endpoint-project associations test case.
 
         """
         r = self.get('/OS-EP-FILTER/endpoints/%(endpoint_id)s/projects' %
@@ -177,7 +181,7 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
                      expected_status=200)
         self.assertValidProjectListResponse(r, expected_length=0)
 
-    def test_list_projects_for_endpoint_noendpoint(self):
+    def test_list_projects_associated_with_invalid_endpoint(self):
         """GET /OS-EP-FILTER/endpoints/{endpoint_id}/projects
 
         Invalid endpoint id test case.
@@ -187,20 +191,7 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
                  {'endpoint_id': uuid.uuid4().hex},
                  expected_status=404)
 
-    def test_list_projects_for_endpoint_assoc(self):
-        """GET /OS-EP-FILTER/endpoints/{endpoint_id}/projects success
-
-        Associate default project and endpoint, then get it.
-
-        """
-        self.put(self.default_request_url)
-        r = self.get('/OS-EP-FILTER/endpoints/%(endpoint_id)s/projects' %
-                     {'endpoint_id': self.endpoint_id},
-                     expected_status=200)
-        self.assertValidProjectListResponse(r, self.default_domain_project)
-
-    # DELETE
-    def test_remove_endpoint_project_assoc(self):
+    def test_remove_endpoint_project_association(self):
         """DELETE /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Valid project id and endpoint id test case.
@@ -209,11 +200,11 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         self.put(self.default_request_url)
         self.delete('/OS-EP-FILTER/projects/%(project_id)s'
                     '/endpoints/%(endpoint_id)s' % {
-                    'project_id': self.default_domain_project_id,
-                    'endpoint_id': self.endpoint_id},
+                        'project_id': self.default_domain_project_id,
+                        'endpoint_id': self.endpoint_id},
                     expected_status=204)
 
-    def test_remove_endpoint_project_assoc_noproj(self):
+    def test_remove_endpoint_project_association_with_invalid_project(self):
         """DELETE /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid project id test case.
@@ -222,12 +213,12 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         self.put(self.default_request_url)
         self.delete('/OS-EP-FILTER/projects/%(project_id)s'
                     '/endpoints/%(endpoint_id)s' % {
-                    'project_id': uuid.uuid4().hex,
-                    'endpoint_id': self.endpoint_id},
+                        'project_id': uuid.uuid4().hex,
+                        'endpoint_id': self.endpoint_id},
                     body='',
                     expected_status=404)
 
-    def test_remove_endpoint_project_assoc_noendp(self):
+    def test_remove_endpoint_project_association_with_invalid_endpoint(self):
         """DELETE /OS-EP-FILTER/projects/{project_id}/endpoints/{endpoint_id}
 
         Invalid endpoint id test case.
@@ -236,17 +227,43 @@ class AssociateEndpointProjectFilterCRUDTestCase(TestExtensionCase):
         self.put(self.default_request_url)
         self.delete('/OS-EP-FILTER/projects/%(project_id)s'
                     '/endpoints/%(endpoint_id)s' % {
-                    'project_id': self.default_domain_project_id,
-                    'endpoint_id': uuid.uuid4().hex},
+                        'project_id': self.default_domain_project_id,
+                        'endpoint_id': uuid.uuid4().hex},
                     body='',
                     expected_status=404)
 
+    def test_endpoint_project_association_cleanup_when_project_deleted(self):
+        self.put(self.default_request_url)
+        association_url = '/OS-EP-FILTER/endpoints/%(endpoint_id)s/projects' % {
+            'endpoint_id': self.endpoint_id}
+        r = self.get(association_url, expected_status=200)
+        self.assertValidProjectListResponse(r, expected_length=1)
 
-class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
-    """Test OS-EP-FILTER catalog filtering extension."""
+        self.delete('/projects/%(project_id)s' % {
+            'project_id': self.default_domain_project_id})
 
-    def test_default_project_id_scoped_token_with_user_id_ep_filter(self):
-        # create a second project to work with
+        r = self.get(association_url, expected_status=200)
+        self.assertValidProjectListResponse(r, expected_length=0)
+
+    def test_endpoint_project_association_cleanup_when_endpoint_deleted(self):
+        self.put(self.default_request_url)
+        association_url = '/OS-EP-FILTER/projects/%(project_id)s/endpoints' % {
+            'project_id': self.default_domain_project_id}
+        r = self.get(association_url, expected_status=200)
+        self.assertValidEndpointListResponse(r, expected_length=1)
+
+        self.delete('/endpoints/%(endpoint_id)s' % {
+            'endpoint_id': self.endpoint_id})
+
+        r = self.get(association_url, expected_status=200)
+        self.assertValidEndpointListResponse(r, expected_length=0)
+
+
+class EndpointFilterTokenRequestTestCase(TestExtensionCase):
+
+    def test_project_scoped_token_using_endpoint_filter(self):
+        """Verify endpoints from project scoped token filtered.""" 
+        # create a project to work with
         ref = self.new_project_ref(domain_id=self.domain_id)
         r = self.post('/projects', body={'project': ref})
         project = self.assertValidProjectResponse(r, ref)
@@ -268,8 +285,8 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         # add one endpoint to the project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': project['id'],
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': project['id'],
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=204)
 
@@ -285,14 +302,13 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
             ep_filter_assoc=1)
         self.assertEqual(r.result['token']['project']['id'], project['id'])
 
-    def test_implicit_project_id_scoped_token_with_user_id_ep_filter(self):
-        # attempt to authenticate without requesting a project
-
+    def test_default_scoped_token_using_endpoint_filter(self):
+        """Verify endpoints from default scoped token filtered."""
         # add one endpoint to default project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': self.project['id'],
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': self.project['id'],
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=204)
 
@@ -309,8 +325,14 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         self.assertEqual(r.result['token']['project']['id'],
                          self.project['id'])
 
-    def test_default_project_id_scoped_token_ep_filter_no_catalog(self):
-        # create a second project to work with
+    def test_project_scoped_token_with_no_catalog_using_endpoint_filter(self): 
+        """Verify endpoint filter when project scoped token returns no catalog.
+
+        Test that the project scoped token response is valid for a given
+        endpoint-project association when no service catalog is returned.
+
+        """
+        # create a project to work with
         ref = self.new_project_ref(domain_id=self.domain_id)
         r = self.post('/projects', body={'project': ref})
         project = self.assertValidProjectResponse(r, ref)
@@ -332,8 +354,8 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         # add one endpoint to the project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': project['id'],
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': project['id'],
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=204)
 
@@ -349,14 +371,18 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
             ep_filter_assoc=1)
         self.assertEqual(r.result['token']['project']['id'], project['id'])
 
-    def test_implicit_project_id_scoped_token_ep_filter_no_catalog(self):
-        # attempt to authenticate without requesting a project
+    def test_default_scoped_token_with_no_catalog_using_endpoint_filter(self):
+        """Verify endpoint filter when default scoped token returns no catalog.
 
+        Test that the default project scoped token response is valid for a
+        given endpoint-project association when no service catalog is returned.
+
+        """
         # add one endpoint to default project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': self.project['id'],
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': self.project['id'],
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=204)
 
@@ -373,8 +399,14 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         self.assertEqual(r.result['token']['project']['id'],
                          self.project['id'])
 
-    def test_default_project_id_scoped_token_ep_filter_full_catalog(self):
-        # create a second project to work with
+    def test_project_scoped_token_with_no_endpoint_project_association(self):
+        """Verify endpoint filter when no endpoint-project association.
+
+        Test that the project scoped token response is valid when there are
+        no endpoint-project associations defined.
+
+        """
+        # create a project to work with
         ref = self.new_project_ref(domain_id=self.domain_id)
         r = self.post('/projects', body={'project': ref})
         project = self.assertValidProjectResponse(r, ref)
@@ -404,9 +436,13 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
             endpoint_filter=True)
         self.assertEqual(r.result['token']['project']['id'], project['id'])
 
-    def test_implicit_project_id_scoped_token_ep_filter_full_catalog(self):
-        # attempt to authenticate without requesting a project
+    def test_default_scoped_token_with_no_endpoint_project_association(self):
+        """Verify endpoint filter when no endpoint-project association.
 
+        Test that the default project scoped token response is valid when
+        there are no endpoint-project associations defined.
+        
+        """
         auth_data = self.build_authentication_request(
             user_id=self.user['id'],
             password=self.user['password'],
@@ -419,14 +455,13 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         self.assertEqual(r.result['token']['project']['id'],
                          self.project['id'])
 
-    def test_implicit_project_id_scoped_token_handling_bad_reference(self):
-        # handling the case with an endpoint that is not associate with
-
+    def test_invalid_endpoint_project_association(self):
+        """Verify an invalid endpoint-project association is handled."""
         # add first endpoint to default project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': self.project['id'],
-                 'endpoint_id': self.endpoint_id},
+                     'project_id': self.project['id'],
+                     'endpoint_id': self.endpoint_id},
                  body='',
                  expected_status=204)
 
@@ -441,8 +476,8 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
         # add second endpoint to default project
         self.put('/OS-EP-FILTER/projects/%(project_id)s'
                  '/endpoints/%(endpoint_id)s' % {
-                 'project_id': self.project['id'],
-                 'endpoint_id': self.endpoint_id2},
+                     'project_id': self.project['id'],
+                     'endpoint_id': self.endpoint_id2},
                  body='',
                  expected_status=204)
 
@@ -463,3 +498,42 @@ class AssociateProjectEndpointFilterTokenRequestTestCase(TestExtensionCase):
             ep_filter_assoc=1)
         self.assertEqual(r.result['token']['project']['id'],
                          self.project['id'])
+
+    def test_disabled_endpoint(self):
+        """Test that a disabled endpoint is handled."""
+        # Add an enabled endpoint to the default project
+        self.put('/OS-EP-FILTER/projects/%(project_id)s'
+                 '/endpoints/%(endpoint_id)s' % {
+                     'project_id': self.project['id'],
+                     'endpoint_id': self.endpoint_id},
+                 expected_status=204)
+
+        # Add a disabled endpoint to the default project.
+
+        # Create a disabled endpoint that's like the enabled one.
+        disabled_endpoint_ref = copy.copy(self.endpoint)
+        disabled_endpoint_id = uuid.uuid4().hex
+        disabled_endpoint_ref.update({
+            'id': disabled_endpoint_id,
+            'enabled': False,
+            'interface': 'internal'
+        })
+        self.catalog_api.create_endpoint(disabled_endpoint_id,
+                                         disabled_endpoint_ref)
+
+        self.put('/OS-EP-FILTER/projects/%(project_id)s'
+                 '/endpoints/%(endpoint_id)s' % {
+                     'project_id': self.project['id'],
+                     'endpoint_id': disabled_endpoint_id},
+                 expected_status=204)
+
+        # Authenticate to get token with catalog
+        auth_data = self.build_authentication_request(
+            user_id=self.user['id'],
+            password=self.user['password'],
+            project_id=self.project['id'])
+        r = self.post('/auth/tokens', body=auth_data)
+
+        endpoints = r.result['token']['catalog'][0]['endpoints']
+        endpoint_ids = [ep['id'] for ep in endpoints]
+        self.assertEqual([self.endpoint_id], endpoint_ids)
