@@ -15,6 +15,7 @@ import datetime
 import uuid
 
 import mock
+from testtools import matchers
 
 from keystone.common import dependency
 from keystone import config
@@ -79,12 +80,14 @@ def _matches(event, token_values):
 
     # The token has two attributes that can match the domain_id
     if event.domain_id is not None:
-        dom_id_matched = False
-        for attribute_name in ['user_domain_id', 'project_domain_id']:
+        for attribute_name in ['identity_domain_id', 'assignment_domain_id']:
             if event.domain_id == token_values[attribute_name]:
-                dom_id_matched = True
                 break
-        if not dom_id_matched:
+        else:
+            return False
+
+    if event.domain_scope_id is not None:
+        if event.domain_scope_id != token_values['assignment_domain_id']:
             return False
 
     # If any one check does not match, the while token does
@@ -102,12 +105,10 @@ def _matches(event, token_values):
 
     if event.role_id is not None:
         roles = token_values['roles']
-        role_found = False
         for role in roles:
             if event.role_id == role:
-                role_found = True
                 break
-        if not role_found:
+        else:
             return False
     if token_values['issued_at'] > event.issued_before:
         return False
@@ -167,9 +168,20 @@ class RevokeTests(object):
         # 'synchronize' should now be exercised and remove old expired events
         mock_utcnow.return_value = now_plus_2h
         self.revoke_api.revoke_by_expiration(_new_id(), now_plus_2h)
-        #should no longer throw an exception
+        # should no longer throw an exception
         self.revoke_api.check_token(token_values)
 
+    def test_revoke_by_expiration_project_and_domain_fails(self):
+        user_id = _new_id()
+        expires_at = timeutils.isotime(_future_time(), subsecond=True)
+        domain_id = _new_id()
+        project_id = _new_id()
+        self.assertThat(
+            lambda: self.revoke_api.revoke_by_expiration(
+                user_id, expires_at, domain_id=domain_id,
+                project_id=project_id),
+            matchers.raises(exception.UnexpectedError))
+
 
 class SqlRevokeTests(test_backend_sql.SqlTests, RevokeTests):
     def config_overrides(self):
@@ -259,10 +271,13 @@ class RevokeTreeTests(tests.TestCase):
         return self.tree.add_event(
             model.RevokeEvent(user_id=user_id))
 
-    def _revoke_by_expiration(self, user_id, expires_at):
+    def _revoke_by_expiration(self, user_id, expires_at, project_id=None,
+                              domain_id=None):
         event = self.tree.add_event(
             model.RevokeEvent(user_id=user_id,
-                              expires_at=expires_at))
+                              expires_at=expires_at,
+                              project_id=project_id,
+                              domain_id=domain_id))
         self.events.append(event)
         return event
 
@@ -297,6 +312,10 @@ class RevokeTreeTests(tests.TestCase):
         self.events.append(event)
         return event
 
+    def _revoke_by_domain(self, domain_id):
+        event = self.tree.add_event(model.RevokeEvent(domain_id=domain_id))
+        self.events.append(event)
+
     def _user_field_test(self, field_name):
         user_id = _new_id()
         event = self._revoke_by_user(user_id)
@@ -327,7 +346,7 @@ class RevokeTreeTests(tests.TestCase):
         event = self._revoke_by_expiration(user_id, future_time)
         token_data_1 = _sample_blank_token()
         token_data_1['user_id'] = user_id
-        token_data_1['expires_at'] = future_time
+        token_data_1['expires_at'] = future_time.replace(microsecond=0)
         self._assertTokenRevoked(token_data_1)
 
         token_data_2 = _sample_blank_token()
@@ -340,6 +359,40 @@ class RevokeTreeTests(tests.TestCase):
         self.removeEvent(event)
         self._assertTokenNotRevoked(token_data_1)
 
+    def test_by_user_project(self):
+        # When a user has a project-scoped token and the project-scoped token
+        # is revoked then the token is revoked.
+
+        user_id = _new_id()
+        project_id = _new_id()
+
+        future_time = _future_time()
+
+        token_data = _sample_blank_token()
+        token_data['user_id'] = user_id
+        token_data['project_id'] = project_id
+        token_data['expires_at'] = future_time.replace(microsecond=0)
+
+        self._revoke_by_expiration(user_id, future_time, project_id=project_id)
+        self._assertTokenRevoked(token_data)
+
+    def test_by_user_domain(self):
+        # When a user has a domain-scoped token and the domain-scoped token
+        # is revoked then the token is revoked.
+
+        user_id = _new_id()
+        domain_id = _new_id()
+
+        future_time = _future_time()
+
+        token_data = _sample_blank_token()
+        token_data['user_id'] = user_id
+        token_data['assignment_domain_id'] = domain_id
+        token_data['expires_at'] = future_time.replace(microsecond=0)
+
+        self._revoke_by_expiration(user_id, future_time, domain_id=domain_id)
+        self._assertTokenRevoked(token_data)
+
     def removeEvent(self, event):
         self.events.remove(event)
         self.tree.remove_event(event)
@@ -407,6 +460,57 @@ class RevokeTreeTests(tests.TestCase):
         token_data['project_id'] = project_id
         self._assertTokenRevoked(token_data)
 
+    def test_by_domain_user(self):
+        # If revoke a domain, then a token for a user in the domain is revoked
+
+        user_id = _new_id()
+        domain_id = _new_id()
+
+        token_data = _sample_blank_token()
+        token_data['user_id'] = user_id
+        token_data['identity_domain_id'] = domain_id
+
+        self._revoke_by_domain(domain_id)
+
+        self._assertTokenRevoked(token_data)
+
+    def test_by_domain_project(self):
+        # If revoke a domain, then a token scoped to a project in the domain
+        # is revoked.
+
+        user_id = _new_id()
+        user_domain_id = _new_id()
+
+        project_id = _new_id()
+        project_domain_id = _new_id()
+
+        token_data = _sample_blank_token()
+        token_data['user_id'] = user_id
+        token_data['identity_domain_id'] = user_domain_id
+        token_data['project_id'] = project_id
+        token_data['assignment_domain_id'] = project_domain_id
+
+        self._revoke_by_domain(project_domain_id)
+
+        self._assertTokenRevoked(token_data)
+
+    def test_by_domain_domain(self):
+        # If revoke a domain, then a token scoped to the domain is revoked.
+
+        user_id = _new_id()
+        user_domain_id = _new_id()
+
+        domain_id = _new_id()
+
+        token_data = _sample_blank_token()
+        token_data['user_id'] = user_id
+        token_data['identity_domain_id'] = user_domain_id
+        token_data['assignment_domain_id'] = domain_id
+
+        self._revoke_by_domain(domain_id)
+
+        self._assertTokenRevoked(token_data)
+
     def _assertEmpty(self, collection):
         return self.assertEqual(0, len(collection), "collection not empty")
 
@@ -441,11 +545,23 @@ class RevokeTreeTests(tests.TestCase):
     def test_cleanup(self):
         events = self.events
         self._assertEmpty(self.tree.revoke_map)
+        expiry_base_time = _future_time()
         for i in range(0, 10):
             events.append(
                 self._revoke_by_user(_new_id()))
+
+            args = (_new_id(),
+                    expiry_base_time + datetime.timedelta(seconds=i))
             events.append(
-                self._revoke_by_expiration(_new_id(), _future_time()))
+                self._revoke_by_expiration(*args))
+
+            self.assertEqual(i + 2, len(self.tree.revoke_map
+                                        ['trust_id=*']
+                                        ['consumer_id=*']
+                                        ['access_token_id=*']),
+                             'adding %s to %s' % (args,
+                                                  self.tree.revoke_map))
+
             events.append(
                 self._revoke_by_project_role_assignment(_new_id(), _new_id()))
             events.append(
