@@ -31,6 +31,7 @@ To run these tests against a live database:
    all data will be lost.
 """
 
+from oslo.db.sqlalchemy import utils
 
 from keystone.contrib import endpoint_filter
 from keystone.contrib import example
@@ -168,8 +169,19 @@ class FederationExtension(test_sql_upgrade.SqlMigrateBase):
         self.assertTableColumns(self.mapping,
                                 ['id', 'rules'])
 
+        federation_protocol = utils.get_table(
+            self.engine,
+            'federation_protocol')
+        with self.engine.begin() as conn:
+            conn.execute(federation_protocol.insert(), id=0, idp_id=1)
+            self.upgrade(3, repository=self.repo_path)
+            federation_protocol = utils.get_table(
+                self.engine,
+                'federation_protocol')
+            self.assertFalse(federation_protocol.c.mapping_id.nullable)
+
     def test_downgrade(self):
-        self.upgrade(2, repository=self.repo_path)
+        self.upgrade(3, repository=self.repo_path)
         self.assertTableColumns(self.identity_provider,
                                 ['id', 'enabled', 'description'])
         self.assertTableColumns(self.federation_protocol,
@@ -177,6 +189,12 @@ class FederationExtension(test_sql_upgrade.SqlMigrateBase):
         self.assertTableColumns(self.mapping,
                                 ['id', 'rules'])
 
+        self.downgrade(2, repository=self.repo_path)
+        federation_protocol = utils.get_table(
+            self.engine,
+            'federation_protocol')
+        self.assertTrue(federation_protocol.c.mapping_id.nullable)
+
         self.downgrade(0, repository=self.repo_path)
         self.assertTableDoesNotExist(self.identity_provider)
         self.assertTableDoesNotExist(self.federation_protocol)
