@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 # Copyright 2013 IBM Corp.
 #
@@ -52,68 +50,72 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         # Ensure that test_v3.RestfulTestCase doesn't load its own
         # sample data, which would make checking the results of our
         # tests harder
-        super(IdentityTestProtectedCase, self).setUp(load_sample_data=False)
-        # Start by creating a couple of domains
-        self.domainA = self.new_domain_ref()
-        self.identity_api.create_domain(self.domainA['id'], self.domainA)
-        self.domainB = self.new_domain_ref()
-        self.identity_api.create_domain(self.domainB['id'], self.domainB)
-        self.domainC = self.new_domain_ref()
-        self.domainC['enabled'] = False
-        self.identity_api.create_domain(self.domainC['id'], self.domainC)
-
-        # Now create some users, one in domainA and two of them in domainB
-        self.user1 = self.new_user_ref(domain_id=self.domainA['id'])
-        self.user1['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user1['id'], self.user1)
-
-        self.user2 = self.new_user_ref(domain_id=self.domainB['id'])
-        self.user2['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user2['id'], self.user2)
-
-        self.user3 = self.new_user_ref(domain_id=self.domainB['id'])
-        self.user3['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.user3['id'], self.user3)
-
-        self.group1 = self.new_group_ref(domain_id=self.domainA['id'])
-        self.identity_api.create_group(self.group1['id'], self.group1)
-
-        self.group2 = self.new_group_ref(domain_id=self.domainA['id'])
-        self.identity_api.create_group(self.group2['id'], self.group2)
-
-        self.group3 = self.new_group_ref(domain_id=self.domainB['id'])
-        self.identity_api.create_group(self.group3['id'], self.group3)
-
-        self.role = self.new_role_ref()
-        self.identity_api.create_role(self.role['id'], self.role)
-        self.role1 = self.new_role_ref()
-        self.identity_api.create_role(self.role1['id'], self.role1)
-        self.identity_api.create_grant(self.role['id'],
-                                       user_id=self.user1['id'],
-                                       domain_id=self.domainA['id'])
-        self.identity_api.create_grant(self.role['id'],
-                                       user_id=self.user2['id'],
-                                       domain_id=self.domainA['id'])
-        self.identity_api.create_grant(self.role1['id'],
-                                       user_id=self.user1['id'],
-                                       domain_id=self.domainA['id'])
+        super(IdentityTestProtectedCase, self).setUp()
 
         # Initialize the policy engine and allow us to write to a temp
         # file in each test to create the policies
-        self.orig_policy_file = CONF.policy_file
+        self.addCleanup(rules.reset)
         rules.reset()
         _unused, self.tmpfilename = tempfile.mkstemp()
-        self.opt(policy_file=self.tmpfilename)
+        self.config_fixture.config(policy_file=self.tmpfilename)
 
         # A default auth request we can use - un-scoped user token
         self.auth = self.build_authentication_request(
             user_id=self.user1['id'],
             password=self.user1['password'])
 
-    def tearDown(self):
-        super(IdentityTestProtectedCase, self).tearDown()
-        rules.reset()
-        self.opt(policy_file=self.orig_policy_file)
+    def load_sample_data(self):
+        self._populate_default_domain()
+        # Start by creating a couple of domains
+        self.domainA = self.new_domain_ref()
+        self.assignment_api.create_domain(self.domainA['id'], self.domainA)
+        self.domainB = self.new_domain_ref()
+        self.assignment_api.create_domain(self.domainB['id'], self.domainB)
+        self.domainC = self.new_domain_ref()
+        self.domainC['enabled'] = False
+        self.assignment_api.create_domain(self.domainC['id'], self.domainC)
+
+        # Now create some users, one in domainA and two of them in domainB
+        self.user1 = self.new_user_ref(domain_id=self.domainA['id'])
+        password = uuid.uuid4().hex
+        self.user1['password'] = password
+        self.user1 = self.identity_api.create_user(self.user1)
+        self.user1['password'] = password
+
+        self.user2 = self.new_user_ref(domain_id=self.domainB['id'])
+        password = uuid.uuid4().hex
+        self.user2['password'] = password
+        self.user2 = self.identity_api.create_user(self.user2)
+        self.user2['password'] = password
+
+        self.user3 = self.new_user_ref(domain_id=self.domainB['id'])
+        password = uuid.uuid4().hex
+        self.user3['password'] = password
+        self.user3 = self.identity_api.create_user(self.user3)
+        self.user3['password'] = password
+
+        self.group1 = self.new_group_ref(domain_id=self.domainA['id'])
+        self.group1 = self.identity_api.create_group(self.group1)
+
+        self.group2 = self.new_group_ref(domain_id=self.domainA['id'])
+        self.group2 = self.identity_api.create_group(self.group2)
+
+        self.group3 = self.new_group_ref(domain_id=self.domainB['id'])
+        self.group3 = self.identity_api.create_group(self.group3)
+
+        self.role = self.new_role_ref()
+        self.assignment_api.create_role(self.role['id'], self.role)
+        self.role1 = self.new_role_ref()
+        self.assignment_api.create_role(self.role1['id'], self.role1)
+        self.assignment_api.create_grant(self.role['id'],
+                                         user_id=self.user1['id'],
+                                         domain_id=self.domainA['id'])
+        self.assignment_api.create_grant(self.role['id'],
+                                         user_id=self.user2['id'],
+                                         domain_id=self.domainA['id'])
+        self.assignment_api.create_grant(self.role1['id'],
+                                         user_id=self.user1['id'],
+                                         domain_id=self.domainA['id'])
 
     def _get_id_list_from_ref_list(self, ref_list):
         result_list = []
@@ -129,6 +131,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /users (unprotected)
 
         Test Plan:
+
         - Update policy so api is unprotected
         - Use an un-scoped token to make sure we can get back all
           the users independent of domain
@@ -145,6 +148,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /users?domain_id=mydomain (filtered)
 
         Test Plan:
+
         - Update policy so api is unprotected
         - Use an un-scoped token to make sure we can filter the
           users by domainB, getting back the 2 users in that domain
@@ -162,6 +166,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /users/{id} (match payload)
 
         Test Plan:
+
         - Update policy to protect api by user_id
         - List users with user_id of user1 as filter, to check that
           this will correctly match user_id in the flattened
@@ -175,12 +180,13 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         self._set_policy(new_policy)
         url_by_name = '/users/%s' % self.user1['id']
         r = self.get(url_by_name, auth=self.auth)
-        self.assertEquals(self.user1['id'], r.result['user']['id'])
+        self.assertEqual(self.user1['id'], r.result['user']['id'])
 
     def test_get_user_protected_match_target(self):
         """GET /users/{id} (match target)
 
         Test Plan:
+
         - Update policy to protect api by domain_id
         - Try and read a user who is in DomainB with a token scoped
           to Domain A - this should fail
@@ -202,7 +208,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
 
         url_by_name = '/users/%s' % self.user1['id']
         r = self.get(url_by_name, auth=self.auth)
-        self.assertEquals(self.user1['id'], r.result['user']['id'])
+        self.assertEqual(self.user1['id'], r.result['user']['id'])
 
         url_by_name = '/users/%s' % uuid.uuid4().hex
         r = self.get(url_by_name, auth=self.auth,
@@ -212,6 +218,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """DELETE /domains/{id}/users/{id}/roles/{id} (match target)
 
         Test Plan:
+
         - Update policy to protect api by domain_id of entities in
           the grant
         - Try and delete the existing grant that has a user who is
@@ -250,6 +257,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /users?domain_id=mydomain (protected)
 
         Test Plan:
+
         - Update policy to protect api by domain_id
         - List groups using a token scoped to domainA with a filter
           specifying domainA - we should only get back the one user
@@ -268,7 +276,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         r = self.get(url_by_name, auth=self.auth)
         # We should only get back one user, the one in DomainA
         id_list = self._get_id_list_from_ref_list(r.result.get('users'))
-        self.assertEqual(len(id_list), 1)
+        self.assertEqual(1, len(id_list))
         self.assertIn(self.user1['id'], id_list)
 
         # Now try for domainB, which should fail
@@ -280,6 +288,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /groups?domain_id=mydomain (protected)
 
         Test Plan:
+
         - Update policy to protect api by domain_id
         - List groups using a token scoped to domainA and make sure
           we only get back the two groups that are in domainA
@@ -297,7 +306,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         r = self.get(url_by_name, auth=self.auth)
         # We should only get back two groups, the ones in DomainA
         id_list = self._get_id_list_from_ref_list(r.result.get('groups'))
-        self.assertEqual(len(id_list), 2)
+        self.assertEqual(2, len(id_list))
         self.assertIn(self.group1['id'], id_list)
         self.assertIn(self.group2['id'], id_list)
 
@@ -310,6 +319,7 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         """GET /groups?domain_id=mydomain&name=myname (protected)
 
         Test Plan:
+
         - Update policy to protect api by domain_id
         - List groups using a token scoped to domainA with a filter
           specifying both domainA and the name of group.
@@ -329,62 +339,9 @@ class IdentityTestProtectedCase(test_v3.RestfulTestCase):
         # We should only get back one user, the one in DomainA that matches
         # the name supplied
         id_list = self._get_id_list_from_ref_list(r.result.get('groups'))
-        self.assertEqual(len(id_list), 1)
+        self.assertEqual(1, len(id_list))
         self.assertIn(self.group2['id'], id_list)
 
-    def test_list_filtered_domains(self):
-        """GET /domains?enabled=0
-
-        Test Plan:
-        - Update policy for no protection on api
-        - Filter by the 'enabled' boolean to get disabled domains, which
-          should return just domainC
-        - Try the filter using different ways of specifying 'true'
-          to test that our handling of booleans in filter matching is
-          correct
-
-        """
-        new_policy = {"identity:list_domains": []}
-        self._set_policy(new_policy)
-        r = self.get('/domains?enabled=0', auth=self.auth)
-        id_list = self._get_id_list_from_ref_list(r.result.get('domains'))
-        self.assertEqual(len(id_list), 1)
-        self.assertIn(self.domainC['id'], id_list)
-
-        # Now try a few ways of specifying 'true' when we should get back
-        # the other two domains, plus the default domain
-        r = self.get('/domains?enabled=1', auth=self.auth)
-        id_list = self._get_id_list_from_ref_list(r.result.get('domains'))
-        self.assertEqual(len(id_list), 3)
-        self.assertIn(self.domainA['id'], id_list)
-        self.assertIn(self.domainB['id'], id_list)
-        self.assertIn(DEFAULT_DOMAIN_ID, id_list)
-
-        r = self.get('/domains?enabled', auth=self.auth)
-        id_list = self._get_id_list_from_ref_list(r.result.get('domains'))
-        self.assertEqual(len(id_list), 3)
-        self.assertIn(self.domainA['id'], id_list)
-        self.assertIn(self.domainB['id'], id_list)
-        self.assertIn(DEFAULT_DOMAIN_ID, id_list)
-
-    def test_multiple_filters(self):
-        """GET /domains?enabled&name=myname
-
-        Test Plan:
-        - Update policy for no protection on api
-        - Filter by the 'enabled' boolean and name - this should
-          return a single domain
-
-        """
-        new_policy = {"identity:list_domains": []}
-        self._set_policy(new_policy)
-
-        my_url = '/domains?enableds&name=%s' % self.domainA['name']
-        r = self.get(my_url, auth=self.auth)
-        id_list = self._get_id_list_from_ref_list(r.result.get('domains'))
-        self.assertEqual(len(id_list), 1)
-        self.assertIn(self.domainA['id'], id_list)
-
 
 class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
     """Test policy enforcement of the sample v3 cloud policy file."""
@@ -394,11 +351,16 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
 
         The following data is created:
 
-        - Three domains: A, B and admin_domain, and one project
-        - DomainA has users: domain_admin and just_a_user. domain_admin has
-          role 'admin', just_a_user does not
-        - admin_domain has user cloud_admin, with a plain role
-        - domain_admin and just_a_user gave the same roles on the project
+        - Three domains: domainA, domainB and admin_domain
+        - One project, which name is 'project'
+        - domainA has three users: domain_admin_user, project_admin_user and
+          just_a_user:
+
+          - domain_admin_user has role 'admin' on domainA,
+          - project_admin_user has role 'admin' on the project,
+          - just_a_user has a non-admin role on both domainA and the project.
+        - admin_domain has user cloud_admin_user, with an 'admin' role
+          on admin_domain.
 
         We test various api protection rules from the cloud sample policy
         file to make sure the sample is valid and that we correctly enforce it.
@@ -407,13 +369,21 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         # Ensure that test_v3.RestfulTestCase doesn't load its own
         # sample data, which would make checking the results of our
         # tests harder
-        super(IdentityTestv3CloudPolicySample, self).setUp(
-            load_sample_data=False)
+        super(IdentityTestv3CloudPolicySample, self).setUp()
+
+        # Finally, switch to the v3 sample policy file
+        self.addCleanup(rules.reset)
+        rules.reset()
+        self.config_fixture.config(
+            policy_file=tests.dirs.etc('policy.v3cloudsample.json'))
+
+    def load_sample_data(self):
         # Start by creating a couple of domains
+        self._populate_default_domain()
         self.domainA = self.new_domain_ref()
-        self.identity_api.create_domain(self.domainA['id'], self.domainA)
+        self.assignment_api.create_domain(self.domainA['id'], self.domainA)
         self.domainB = self.new_domain_ref()
-        self.identity_api.create_domain(self.domainB['id'], self.domainB)
+        self.assignment_api.create_domain(self.domainB['id'], self.domainB)
         self.admin_domain = {'id': 'admin_domain_id', 'name': 'Admin_domain'}
         self.assignment_api.create_domain(self.admin_domain['id'],
                                           self.admin_domain)
@@ -421,28 +391,36 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         # And our users
         self.cloud_admin_user = self.new_user_ref(
             domain_id=self.admin_domain['id'])
-        self.cloud_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.cloud_admin_user['id'],
-                                      self.cloud_admin_user)
+        password = uuid.uuid4().hex
+        self.cloud_admin_user['password'] = password
+        self.cloud_admin_user = (
+            self.identity_api.create_user(self.cloud_admin_user))
+        self.cloud_admin_user['password'] = password
         self.just_a_user = self.new_user_ref(domain_id=self.domainA['id'])
-        self.just_a_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.just_a_user['id'], self.just_a_user)
+        password = uuid.uuid4().hex
+        self.just_a_user['password'] = password
+        self.just_a_user = self.identity_api.create_user(self.just_a_user)
+        self.just_a_user['password'] = password
         self.domain_admin_user = self.new_user_ref(
             domain_id=self.domainA['id'])
-        self.domain_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.domain_admin_user['id'],
-                                      self.domain_admin_user)
+        password = uuid.uuid4().hex
+        self.domain_admin_user['password'] = password
+        self.domain_admin_user = (
+            self.identity_api.create_user(self.domain_admin_user))
+        self.domain_admin_user['password'] = password
         self.project_admin_user = self.new_user_ref(
             domain_id=self.domainA['id'])
-        self.project_admin_user['password'] = uuid.uuid4().hex
-        self.identity_api.create_user(self.project_admin_user['id'],
-                                      self.project_admin_user)
+        password = uuid.uuid4().hex
+        self.project_admin_user['password'] = password
+        self.project_admin_user = (
+            self.identity_api.create_user(self.project_admin_user))
+        self.project_admin_user['password'] = password
 
         # The admin role and another plain role
         self.admin_role = {'id': uuid.uuid4().hex, 'name': 'admin'}
         self.assignment_api.create_role(self.admin_role['id'], self.admin_role)
         self.role = self.new_role_ref()
-        self.identity_api.create_role(self.role['id'], self.role)
+        self.assignment_api.create_role(self.role['id'], self.role)
 
         # The cloud admin just gets the admin role
         self.assignment_api.create_grant(self.admin_role['id'],
@@ -467,16 +445,6 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
                                          user_id=self.just_a_user['id'],
                                          project_id=self.project['id'])
 
-        # Finally, switch to the v3 sample policy file
-        self.orig_policy_file = CONF.policy_file
-        rules.reset()
-        self.opt(policy_file=tests.etcdir('policy.v3cloudsample.json'))
-
-    def tearDown(self):
-        super(IdentityTestv3CloudPolicySample, self).tearDown()
-        rules.reset()
-        self.opt(policy_file=self.orig_policy_file)
-
     def _stati(self, expected_status):
         # Return the expected return codes for APIs with and without data
         # with any specified status overriding the normal values
@@ -566,8 +534,8 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
                     expected_status=status_no_data)
 
     def test_user_management(self):
-        # First, authentication with a user that does not have the domain
-        # admin role - houldn't be able to do much.
+        # First, authenticate with a user that does not have the domain
+        # admin role - shouldn't be able to do much.
         self.auth = self.build_authentication_request(
             user_id=self.just_a_user['id'],
             password=self.just_a_user['password'],
@@ -576,7 +544,7 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         self._test_user_management(
             self.domainA['id'], expected=exception.ForbiddenAction.code)
 
-        # Now, authentication with a user that does have the domain admin role
+        # Now, authenticate with a user that does have the domain admin role
         self.auth = self.build_authentication_request(
             user_id=self.domain_admin_user['id'],
             password=self.domain_admin_user['password'],
@@ -584,9 +552,19 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
 
         self._test_user_management(self.domainA['id'])
 
+    def test_user_management_by_cloud_admin(self):
+        # Test users management with a cloud admin. This user should
+        # be able to manage users in any domain.
+        self.auth = self.build_authentication_request(
+            user_id=self.cloud_admin_user['id'],
+            password=self.cloud_admin_user['password'],
+            domain_id=self.admin_domain['id'])
+
+        self._test_user_management(self.domainA['id'])
+
     def test_project_management(self):
-        # First, authentication with a user that does not have the project
-        # admin role - houldn't be able to do much.
+        # First, authenticate with a user that does not have the project
+        # admin role - shouldn't be able to do much.
         self.auth = self.build_authentication_request(
             user_id=self.just_a_user['id'],
             password=self.just_a_user['password'],
@@ -600,7 +578,7 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         url = '/users/%s/projects' % self.just_a_user['id']
         self.get(url, auth=self.auth)
 
-        # Now, authentication with a user that does have the domain admin role
+        # Now, authenticate with a user that does have the domain admin role
         self.auth = self.build_authentication_request(
             user_id=self.domain_admin_user['id'],
             password=self.domain_admin_user['password'],
@@ -617,7 +595,7 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         self._test_grants('domains', self.domainA['id'],
                           expected=exception.ForbiddenAction.code)
 
-        # Now, authentication with a user that does have the domain admin role
+        # Now, authenticate with a user that does have the domain admin role
         self.auth = self.build_authentication_request(
             user_id=self.domain_admin_user['id'],
             password=self.domain_admin_user['password'],
@@ -625,6 +603,21 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
 
         self._test_grants('domains', self.domainA['id'])
 
+        # Check that with such a token we cannot modify grants on a
+        # different domain
+        self._test_grants('domains', self.domainB['id'],
+                          expected=exception.ForbiddenAction.code)
+
+    def test_domain_grants_by_cloud_admin(self):
+        # Test domain grants with a cloud admin. This user should be
+        # able to manage roles on any domain.
+        self.auth = self.build_authentication_request(
+            user_id=self.cloud_admin_user['id'],
+            password=self.cloud_admin_user['password'],
+            domain_id=self.admin_domain['id'])
+
+        self._test_grants('domains', self.domainA['id'])
+
     def test_project_grants(self):
         self.auth = self.build_authentication_request(
             user_id=self.just_a_user['id'],
@@ -634,7 +627,8 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
         self._test_grants('projects', self.project['id'],
                           expected=exception.ForbiddenAction.code)
 
-        # Now, authentication with a user that does have the domain admin role
+        # Now, authenticate with a user that does have the project
+        # admin role
         self.auth = self.build_authentication_request(
             user_id=self.project_admin_user['id'],
             password=self.project_admin_user['password'],
@@ -642,6 +636,16 @@ class IdentityTestv3CloudPolicySample(test_v3.RestfulTestCase):
 
         self._test_grants('projects', self.project['id'])
 
+    def test_project_grants_by_domain_admin(self):
+        # Test project grants with a domain admin. This user should be
+        # able to manage roles on any project in its own domain.
+        self.auth = self.build_authentication_request(
+            user_id=self.domain_admin_user['id'],
+            password=self.domain_admin_user['password'],
+            domain_id=self.domainA['id'])
+
+        self._test_grants('projects', self.project['id'])
+
     def test_cloud_admin(self):
         self.auth = self.build_authentication_request(
             user_id=self.domain_admin_user['id'],
