@@ -13,25 +13,30 @@
 import copy
 import uuid
 
-from oslo_utils import timeutils
-from six.moves import range
-
+from keystone.common import config
 from keystone import exception
-from keystone.federation import constants as federation_constants
 from keystone.models import token_model
-from keystone.tests.unit import core
-from keystone.tests.unit import test_token_provider
+from keystone.openstack.common import timeutils
+from keystone.tests import core
+from keystone.tests import test_token_provider
+
+
+CONF = config.CONF
 
 
 class TestKeystoneTokenModel(core.TestCase):
     def setUp(self):
         super(TestKeystoneTokenModel, self).setUp()
+        self.load_backends()
+        self.v2_sample_token = copy.deepcopy(
+            test_token_provider.SAMPLE_V2_TOKEN)
         self.v3_sample_token = copy.deepcopy(
             test_token_provider.SAMPLE_V3_TOKEN)
 
     def test_token_model_v3(self):
         token_data = token_model.KeystoneToken(uuid.uuid4().hex,
                                                self.v3_sample_token)
+        self.assertIs(token_model.V3, token_data.version)
         expires = timeutils.normalize_time(timeutils.parse_isotime(
             self.v3_sample_token['token']['expires_at']))
         issued = timeutils.normalize_time(timeutils.parse_isotime(
@@ -42,9 +47,6 @@ class TestKeystoneTokenModel(core.TestCase):
                          token_data.user_id)
         self.assertEqual(self.v3_sample_token['token']['user']['name'],
                          token_data.user_name)
-        self.assertEqual(
-            self.v3_sample_token['token']['user']['password_expires_at'],
-            token_data.user_password_expires_at)
         self.assertEqual(self.v3_sample_token['token']['user']['domain']['id'],
                          token_data.user_domain_id)
         self.assertEqual(
@@ -56,8 +58,6 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertEqual(
             self.v3_sample_token['token']['project']['domain']['name'],
             token_data.project_domain_name)
-        self.assertEqual(
-            self.v3_sample_token['token']['is_domain'], token_data.is_domain)
         self.assertEqual(self.v3_sample_token['token']['OS-TRUST:trust']['id'],
                          token_data.trust_id)
         self.assertEqual(
@@ -66,7 +66,6 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertEqual(
             self.v3_sample_token['token']['OS-TRUST:trust']['trustee_user_id'],
             token_data.trustee_user_id)
-
         # Project Scoped Token
         self.assertRaises(exception.UnexpectedError, getattr, token_data,
                           'domain_id')
@@ -80,18 +79,12 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertTrue(token_data.project_scoped)
         self.assertTrue(token_data.scoped)
         self.assertTrue(token_data.trust_scoped)
-
-        # by default admin project is True for project scoped tokens
-        self.assertTrue(token_data.is_admin_project)
-
         self.assertEqual(
             [r['id'] for r in self.v3_sample_token['token']['roles']],
             token_data.role_ids)
         self.assertEqual(
             [r['name'] for r in self.v3_sample_token['token']['roles']],
             token_data.role_names)
-
-        # Domain Scoped Token
         token_data.pop('project')
         self.assertFalse(token_data.project_scoped)
         self.assertFalse(token_data.scoped)
@@ -108,43 +101,71 @@ class TestKeystoneTokenModel(core.TestCase):
         self.assertEqual(domain_name, token_data.domain_name)
         self.assertTrue(token_data.domain_scoped)
 
-        token_data['audit_ids'] = [uuid.uuid4().hex]
-        self.assertEqual(token_data.audit_id,
-                         token_data['audit_ids'][0])
-        self.assertEqual(token_data.audit_chain_id,
-                         token_data['audit_ids'][0])
-        token_data['audit_ids'].append(uuid.uuid4().hex)
-        self.assertEqual(token_data.audit_chain_id,
-                         token_data['audit_ids'][1])
-        del token_data['audit_ids']
-        self.assertIsNone(token_data.audit_id)
-        self.assertIsNone(token_data.audit_chain_id)
-
-        # by default admin project is False for domain scoped tokens
-        self.assertFalse(token_data.is_admin_project)
-
-    def test_token_model_v3_federated_user(self):
-        token_data = token_model.KeystoneToken(token_id=uuid.uuid4().hex,
-                                               token_data=self.v3_sample_token)
-        federation_data = {'identity_provider': {'id': uuid.uuid4().hex},
-                           'protocol': {'id': 'saml2'},
-                           'groups': [{'id': uuid.uuid4().hex}
-                                      for x in range(1, 5)]}
-
-        self.assertFalse(token_data.is_federated_user)
-        self.assertEqual([], token_data.federation_group_ids)
-        self.assertIsNone(token_data.federation_protocol_id)
-        self.assertIsNone(token_data.federation_idp_id)
-
-        token_data['user'][federation_constants.FEDERATION] = federation_data
-
-        self.assertTrue(token_data.is_federated_user)
-        self.assertEqual([x['id'] for x in federation_data['groups']],
-                         token_data.federation_group_ids)
-        self.assertEqual(federation_data['protocol']['id'],
-                         token_data.federation_protocol_id)
-        self.assertEqual(federation_data['identity_provider']['id'],
-                         token_data.federation_idp_id)
+    def test_token_model_v2(self):
+        token_data = token_model.KeystoneToken(uuid.uuid4().hex,
+                                               self.v2_sample_token)
+        self.assertIs(token_model.V2, token_data.version)
+        expires = timeutils.normalize_time(timeutils.parse_isotime(
+            self.v2_sample_token['access']['token']['expires']))
+        issued = timeutils.normalize_time(timeutils.parse_isotime(
+            self.v2_sample_token['access']['token']['issued_at']))
+        self.assertEqual(expires, token_data.expires)
+        self.assertEqual(issued, token_data.issued)
+        self.assertEqual(self.v2_sample_token['access']['user']['id'],
+                         token_data.user_id)
+        self.assertEqual(self.v2_sample_token['access']['user']['name'],
+                         token_data.user_name)
+        self.assertEqual(CONF.identity.default_domain_id,
+                         token_data.user_domain_id)
+        self.assertEqual('Default', token_data.user_domain_name)
+        self.assertEqual(CONF.identity.default_domain_id,
+                         token_data.project_domain_id)
+        self.assertEqual('Default',
+                         token_data.project_domain_name)
+        self.assertEqual(self.v2_sample_token['access']['trust']['id'],
+                         token_data.trust_id)
+        self.assertIsNone(token_data.trustor_user_id)
+        self.assertEqual(
+            self.v2_sample_token['access']['trust']['trustee_user_id'],
+            token_data.trustee_user_id)
+        # Project Scoped Token
+        self.assertEqual(
+            self.v2_sample_token['access']['token']['tenant']['id'],
+            token_data.project_id)
+        self.assertEqual(
+            self.v2_sample_token['access']['token']['tenant']['name'],
+            token_data.project_name)
+        self.assertTrue(token_data.project_scoped)
+        self.assertTrue(token_data.scoped)
+        self.assertTrue(token_data.trust_scoped)
+        self.assertEqual(
+            [r['name']
+             for r in self.v2_sample_token['access']['user']['roles']],
+            token_data.role_names)
+        token_data['token'].pop('tenant')
+        self.assertFalse(token_data.scoped)
+        self.assertFalse(token_data.project_scoped)
+        self.assertFalse(token_data.domain_scoped)
+        self.assertRaises(exception.UnexpectedError, getattr, token_data,
+                          'project_id')
+        self.assertRaises(exception.UnexpectedError, getattr, token_data,
+                          'project_name')
+        self.assertRaises(exception.UnexpectedError, getattr, token_data,
+                          'project_domain_id')
+        self.assertRaises(exception.UnexpectedError, getattr, token_data,
+                          'project_domain_id')
+        # No Domain Scoped tokens in V2
+        self.assertRaises(NotImplementedError, getattr, token_data,
+                          'domain_id')
+        self.assertRaises(NotImplementedError, getattr, token_data,
+                          'domain_name')
+        token_data['domain'] = {'id': uuid.uuid4().hex,
+                                'name': uuid.uuid4().hex}
+        self.assertRaises(NotImplementedError, getattr, token_data,
+                          'domain_id')
+        self.assertRaises(NotImplementedError, getattr, token_data,
+                          'domain_name')
+        self.assertFalse(token_data.domain_scoped)
 
     def test_token_model_unknown(self):
         self.assertRaises(exception.UnsupportedTokenVersionException,
@@ -155,16 +176,14 @@ class TestKeystoneTokenModel(core.TestCase):
     def test_token_model_dual_scoped_token(self):
         domain = {'id': uuid.uuid4().hex,
                   'name': uuid.uuid4().hex}
+        self.v2_sample_token['access']['domain'] = domain
         self.v3_sample_token['token']['domain'] = domain
 
+        # V2 Tokens Cannot be domain scoped, this should work
+        token_model.KeystoneToken(token_id=uuid.uuid4().hex,
+                                  token_data=self.v2_sample_token)
+
         self.assertRaises(exception.UnexpectedError,
                           token_model.KeystoneToken,
                           token_id=uuid.uuid4().hex,
                           token_data=self.v3_sample_token)
-
-    def test_token_model_is_admin_project(self):
-        token_data = token_model.KeystoneToken(token_id=uuid.uuid4().hex,
-                                               token_data=self.v3_sample_token)
-
-        token_data['is_admin_project'] = False
-        self.assertFalse(token_data.is_admin_project)
