@@ -12,78 +12,115 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-"""Main entry point into the Token Persistence service."""
+"""Main entry point into the Token persistence service."""
 
 import abc
 import copy
 
-from oslo_log import log
 import six
 
 from keystone.common import cache
+from keystone.common import dependency
 from keystone.common import manager
-import keystone.conf
+from keystone import config
 from keystone import exception
+from keystone.openstack.common import log
+from keystone.openstack.common import timeutils
+from keystone.openstack.common import versionutils
 
 
-CONF = keystone.conf.CONF
+CONF = config.CONF
 LOG = log.getLogger(__name__)
-MEMOIZE = cache.get_memoization_decorator(group='token')
-REVOCATION_MEMOIZE = cache.get_memoization_decorator(group='token',
-                                                     expiration_group='revoke')
+SHOULD_CACHE = cache.should_cache_fn('token')
+
+# NOTE(blk-u): The config options are not available at import time.
+EXPIRATION_TIME = lambda: CONF.token.cache_time
+REVOCATION_CACHE_EXPIRATION_TIME = lambda: CONF.token.revocation_cache_time
 
 
-class PersistenceManager(manager.Manager):
-    """Default pivot point for the Token Persistence backend.
+@dependency.requires('assignment_api', 'identity_api', 'token_provider_api',
+                     'trust_api')
+@dependency.provider('token_api')
+class Manager(manager.Manager):
+    """Default pivot point for the Token backend.
 
     See :mod:`keystone.common.manager.Manager` for more details on how this
     dynamically calls the backend.
 
     """
 
-    driver_namespace = 'keystone.token.persistence'
-    _provides_api = '_token_persistence_manager'
-
     def __init__(self):
-        super(PersistenceManager, self).__init__(CONF.token.driver)
+        super(Manager, self).__init__(CONF.token.driver)
+
+    @versionutils.deprecated(as_of=versionutils.deprecated.JUNO,
+                             in_favor_of='token_provider_api.unique_id',
+                             remove_in=+1,
+                             what='token_api.unique_id')
+    def unique_id(self, token_id):
+        return self.token_provider_api.unique_id(token_id)
+
+    def _assert_valid(self, token_id, token_ref):
+        """Raise TokenNotFound if the token is expired."""
+        current_time = timeutils.normalize_time(timeutils.utcnow())
+        expires = token_ref.get('expires')
+        if not expires or current_time > timeutils.normalize_time(expires):
+            raise exception.TokenNotFound(token_id=token_id)
 
     def get_token(self, token_id):
-        return self._get_token(token_id)
+        if not token_id:
+            # NOTE(morganfainberg): There are cases when the
+            # context['token_id'] will in-fact be None. This also saves
+            # a round-trip to the backend if we don't have a token_id.
+            raise exception.TokenNotFound(token_id='')
+        unique_id = self.token_provider_api.unique_id(token_id)
+        token_ref = self._get_token(unique_id)
+        # NOTE(morganfainberg): Lift expired checking to the manager, there is
+        # no reason to make the drivers implement this check. With caching,
+        # self._get_token could return an expired token. Make sure we behave
+        # as expected and raise TokenNotFound on those instances.
+        self._assert_valid(token_id, token_ref)
+        return token_ref
 
-    @MEMOIZE
+    @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
+                        expiration_time=EXPIRATION_TIME)
     def _get_token(self, token_id):
         # Only ever use the "unique" id in the cache key.
         return self.driver.get_token(token_id)
 
     def create_token(self, token_id, data):
+        unique_id = self.token_provider_api.unique_id(token_id)
         data_copy = copy.deepcopy(data)
-        data_copy['id'] = token_id
-        ret = self.driver.create_token(token_id, data_copy)
-        if MEMOIZE.should_cache(ret):
+        data_copy['id'] = unique_id
+        ret = self.driver.create_token(unique_id, data_copy)
+        if SHOULD_CACHE(ret):
             # NOTE(morganfainberg): when doing a cache set, you must pass the
             # same arguments through, the same as invalidate (this includes
             # "self"). First argument is always the value to be cached
-            self._get_token.set(ret, self, token_id)
+            self._get_token.set(ret, self, unique_id)
         return ret
 
     def delete_token(self, token_id):
         if not CONF.token.revoke_by_id:
             return
-        self.driver.delete_token(token_id)
-        self._invalidate_individual_token_cache(token_id)
+        unique_id = self.token_provider_api.unique_id(token_id)
+        self.driver.delete_token(unique_id)
+        self._invalidate_individual_token_cache(unique_id)
         self.invalidate_revocation_list()
 
     def delete_tokens(self, user_id, tenant_id=None, trust_id=None,
                       consumer_id=None):
         if not CONF.token.revoke_by_id:
             return
-        token_list = self.driver.delete_tokens(user_id, tenant_id, trust_id,
-                                               consumer_id)
+        token_list = self.driver._list_tokens(user_id, tenant_id, trust_id,
+                                              consumer_id)
+        self.driver.delete_tokens(user_id, tenant_id, trust_id, consumer_id)
         for token_id in token_list:
-            self._invalidate_individual_token_cache(token_id)
+            unique_id = self.token_provider_api.unique_id(token_id)
+            self._invalidate_individual_token_cache(unique_id)
         self.invalidate_revocation_list()
 
-    @REVOCATION_MEMOIZE
+    @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
+                        expiration_time=REVOCATION_CACHE_EXPIRATION_TIME)
     def list_revoked_tokens(self):
         return self.driver.list_revoked_tokens()
 
@@ -105,7 +142,7 @@ class PersistenceManager(manager.Manager):
         """
         if not CONF.token.revoke_by_id:
             return
-        projects = self.resource_api.list_projects()
+        projects = self.assignment_api.list_projects()
         for project in projects:
             if project['domain_id'] == domain_id:
                 for user_id in self.assignment_api.list_user_ids_for_project(
@@ -162,11 +199,17 @@ class PersistenceManager(manager.Manager):
         # NOTE(morganfainberg): invalidate takes the exact same arguments as
         # the normal method, this means we need to pass "self" in (which gets
         # stripped off).
+
+        # FIXME(morganfainberg): Does this cache actually need to be
+        # invalidated? We maintain a cached revocation list, which should be
+        # consulted before accepting a token as valid.  For now we will
+        # do the explicit individual token invalidation.
         self._get_token.invalidate(self, token_id)
+        self.token_provider_api.invalidate_individual_token_cache(token_id)
 
 
 @six.add_metaclass(abc.ABCMeta)
-class TokenDriverBase(object):
+class Driver(object):
     """Interface description for a Token driver."""
 
     @abc.abstractmethod
@@ -176,7 +219,7 @@ class TokenDriverBase(object):
         :param token_id: identity of the token
         :type token_id: string
         :returns: token_ref
-        :raises keystone.exception.TokenNotFound: If the token doesn't exist.
+        :raises: keystone.exception.TokenNotFound
 
         """
         raise exception.NotImplemented()  # pragma: no cover
@@ -196,6 +239,7 @@ class TokenDriverBase(object):
                 id=token_id,
                 user=user_ref,
                 tenant=tenant_ref,
+                metadata=metadata_ref
             }
 
         :type data: dict
@@ -206,12 +250,12 @@ class TokenDriverBase(object):
 
     @abc.abstractmethod
     def delete_token(self, token_id):
-        """Delete a token by id.
+        """Deletes a token by id.
 
         :param token_id: identity of the token
         :type token_id: string
         :returns: None.
-        :raises keystone.exception.TokenNotFound: If the token doesn't exist.
+        :raises: keystone.exception.TokenNotFound
 
         """
         raise exception.NotImplemented()  # pragma: no cover
@@ -219,7 +263,7 @@ class TokenDriverBase(object):
     @abc.abstractmethod
     def delete_tokens(self, user_id, tenant_id=None, trust_id=None,
                       consumer_id=None):
-        """Delete tokens by user.
+        """Deletes tokens by user.
 
         If the tenant_id is not None, only delete the tokens by user id under
         the specified tenant.
@@ -238,8 +282,8 @@ class TokenDriverBase(object):
         :type trust_id: string
         :param consumer_id: identity of the consumer
         :type consumer_id: string
-        :returns: The tokens that have been deleted.
-        :raises keystone.exception.TokenNotFound: If the token doesn't exist.
+        :returns: None.
+        :raises: keystone.exception.TokenNotFound
 
         """
         if not CONF.token.revoke_by_id:
@@ -252,15 +296,13 @@ class TokenDriverBase(object):
         for token in token_list:
             try:
                 self.delete_token(token)
-            except exception.NotFound:  # nosec
-                # The token is already gone, good.
+            except exception.NotFound:
                 pass
-        return token_list
 
     @abc.abstractmethod
     def _list_tokens(self, user_id, tenant_id=None, trust_id=None,
                      consumer_id=None):
-        """Return a list of current token_id's for a user.
+        """Returns a list of current token_id's for a user
 
         This is effectively a private method only used by the ``delete_tokens``
         method and should not be called by anything outside of the
@@ -281,7 +323,7 @@ class TokenDriverBase(object):
 
     @abc.abstractmethod
     def list_revoked_tokens(self):
-        """Return a list of all revoked tokens.
+        """Returns a list of all revoked tokens
 
         :returns: list of token_id's
 
@@ -290,5 +332,6 @@ class TokenDriverBase(object):
 
     @abc.abstractmethod
     def flush_expired_tokens(self):
-        """Archive or delete tokens that have expired."""
+        """Archive or delete tokens that have expired.
+        """
         raise exception.NotImplemented()  # pragma: no cover
