@@ -1,5 +1,3 @@
-# vim: tabstop=4 shiftwidth=4 softtabstop=4
-
 # Copyright 2012 OpenStack Foundation
 #
 # Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -16,37 +14,62 @@
 
 """Token provider interface."""
 
+import abc
+import datetime
+
+from keystoneclient.common import cms
+import six
 
 from keystone.common import cache
 from keystone.common import dependency
 from keystone.common import manager
 from keystone import config
 from keystone import exception
-from keystone.openstack.common import log as logging
+from keystone.i18n import _
+from keystone.models import token_model
+from keystone.openstack.common import log
 from keystone.openstack.common import timeutils
+from keystone.openstack.common import versionutils
 
 
 CONF = config.CONF
-LOG = logging.getLogger(__name__)
+LOG = log.getLogger(__name__)
 SHOULD_CACHE = cache.should_cache_fn('token')
 
+# NOTE(blk-u): The config options are not available at import time.
+EXPIRATION_TIME = lambda: CONF.token.cache_time
 
 # supported token versions
-V2 = 'v2.0'
-V3 = 'v3.0'
-VERSIONS = frozenset([V2, V3])
+V2 = token_model.V2
+V3 = token_model.V3
+VERSIONS = token_model.VERSIONS
 
 # default token providers
 PKI_PROVIDER = 'keystone.token.providers.pki.Provider'
+PKIZ_PROVIDER = 'keystone.token.providers.pkiz.Provider'
 UUID_PROVIDER = 'keystone.token.providers.uuid.Provider'
 
-
-class UnsupportedTokenVersionException(Exception):
-    """Token version is unrecognizable or unsupported."""
-    pass
+_FORMAT_TO_PROVIDER = {
+    'PKI': PKI_PROVIDER,
+    # should not support new options, but PKIZ keeps the option consistent
+    'PKIZ': PKIZ_PROVIDER,
+    'UUID': UUID_PROVIDER
+}
 
 
-@dependency.requires('token_api')
+def default_expire_time():
+    """Determine when a fresh token should expire.
+
+    Expiration time varies based on configuration (see ``[token] expiration``).
+
+    :returns: a naive UTC datetime.datetime object
+
+    """
+    expire_delta = datetime.timedelta(seconds=CONF.token.expiration)
+    return timeutils.utcnow() + expire_delta
+
+
+@dependency.optional('revoke_api')
 @dependency.provider('token_provider_api')
 class Manager(manager.Manager):
     """Default pivot point for the token provider backend.
@@ -56,6 +79,10 @@ class Manager(manager.Manager):
 
     """
 
+    V2 = V2
+    V3 = V3
+    VERSIONS = VERSIONS
+
     @classmethod
     def get_token_provider(cls):
         """Return package path to the configured token provider.
@@ -69,42 +96,42 @@ class Manager(manager.Manager):
         ``provider`` instead.
 
         """
-        if CONF.token.provider is not None:
-            # NOTE(gyee): we are deprecating CONF.signing.token_format. This
-            # code is to ensure the token provider configuration agrees with
-            # CONF.signing.token_format.
-            if (CONF.signing.token_format and
-                    ((CONF.token.provider == PKI_PROVIDER and
-                        CONF.signing.token_format != 'PKI') or
-                        (CONF.token.provider == UUID_PROVIDER and
-                            CONF.signing.token_format != 'UUID'))):
-                raise exception.UnexpectedError(
-                    _('keystone.conf [signing] token_format (deprecated) '
-                      'conflicts with keystone.conf [token] provider'))
-            return CONF.token.provider
-        else:
-            if not CONF.signing.token_format:
-                # No token provider and no format, so use default (PKI)
-                return PKI_PROVIDER
 
-            msg = _('keystone.conf [signing] token_format is deprecated in '
-                    'favor of keystone.conf [token] provider')
-            if CONF.signing.token_format == 'PKI':
-                LOG.warning(msg)
-                return PKI_PROVIDER
-            elif CONF.signing.token_format == 'UUID':
-                LOG.warning(msg)
-                return UUID_PROVIDER
-            else:
+        if CONF.signing.token_format:
+            LOG.warn(_('[signing] token_format is deprecated. '
+                       'Please change to setting the [token] provider '
+                       'configuration value instead'))
+            try:
+
+                mapped = _FORMAT_TO_PROVIDER[CONF.signing.token_format]
+            except KeyError:
                 raise exception.UnexpectedError(
                     _('Unrecognized keystone.conf [signing] token_format: '
                       'expected either \'UUID\' or \'PKI\''))
+            return mapped
+
+        if CONF.token.provider is None:
+            return PKIZ_PROVIDER
+        else:
+            return CONF.token.provider
 
     def __init__(self):
         super(Manager, self).__init__(self.get_token_provider())
 
+    def unique_id(self, token_id):
+        """Return a unique ID for a token.
+
+        The returned value is useful as the primary key of a database table,
+        memcache store, or other lookup table.
+
+        :returns: Given a PKI token, returns it's hashed value. Otherwise,
+                  returns the passed-in value (such as a UUID token ID or an
+                  existing hash).
+        """
+        return cms.cms_hash_token(token_id, mode=CONF.token.hash_algorithm)
+
     def validate_token(self, token_id, belongs_to=None):
-        unique_id = self.token_api.unique_id(token_id)
+        unique_id = self.unique_id(token_id)
         # NOTE(morganfainberg): Ensure we never use the long-form token_id
         # (PKI) as part of the cache_key.
         token = self._validate_token(unique_id)
@@ -112,23 +139,56 @@ class Manager(manager.Manager):
         self._is_valid_token(token)
         return token
 
+    def check_revocation_v2(self, token):
+        try:
+            token_data = token['access']
+        except KeyError:
+            raise exception.TokenNotFound(_('Failed to validate token'))
+
+        if self.revoke_api is not None:
+            token_values = self.revoke_api.model.build_token_values_v2(
+                token_data, CONF.identity.default_domain_id)
+            self.revoke_api.check_token(token_values)
+
     def validate_v2_token(self, token_id, belongs_to=None):
-        unique_id = self.token_api.unique_id(token_id)
+        unique_id = self.unique_id(token_id)
         # NOTE(morganfainberg): Ensure we never use the long-form token_id
         # (PKI) as part of the cache_key.
         token = self._validate_v2_token(unique_id)
+        self.check_revocation_v2(token)
         self._token_belongs_to(token, belongs_to)
         self._is_valid_token(token)
         return token
 
+    def check_revocation_v3(self, token):
+        try:
+            token_data = token['token']
+        except KeyError:
+            raise exception.TokenNotFound(_('Failed to validate token'))
+        if self.revoke_api is not None:
+            token_values = self.revoke_api.model.build_token_values(token_data)
+            self.revoke_api.check_token(token_values)
+
+    def check_revocation(self, token):
+        version = self.driver.get_token_version(token)
+        if version == V2:
+            return self.check_revocation_v2(token)
+        else:
+            return self.check_revocation_v3(token)
+
     def validate_v3_token(self, token_id):
-        unique_id = self.token_api.unique_id(token_id)
+        unique_id = self.unique_id(token_id)
         # NOTE(morganfainberg): Ensure we never use the long-form token_id
         # (PKI) as part of the cache_key.
         token = self._validate_v3_token(unique_id)
         self._is_valid_token(token)
         return token
 
+    @versionutils.deprecated(
+        as_of=versionutils.deprecated.JUNO,
+        what='token_provider_api.check_v2_token',
+        in_favor_of='token_provider_api.validate_v2_token',
+        remove_in=+1)
     def check_v2_token(self, token_id, belongs_to=None):
         """Check the validity of the given V2 token.
 
@@ -139,9 +199,14 @@ class Manager(manager.Manager):
         """
         # NOTE(morganfainberg): Ensure we never use the long-form token_id
         # (PKI) as part of the cache_key.
-        unique_id = self.token_api.unique_id(token_id)
+        unique_id = self.unique_id(token_id)
         self.validate_v2_token(unique_id, belongs_to=belongs_to)
 
+    @versionutils.deprecated(
+        as_of=versionutils.deprecated.JUNO,
+        what='token_provider_api.check_v3_token',
+        in_favor_of='token_provider_api.validate_v3_token',
+        remove_in=+1)
     def check_v3_token(self, token_id):
         """Check the validity of the given V3 token.
 
@@ -151,26 +216,27 @@ class Manager(manager.Manager):
         """
         # NOTE(morganfainberg): Ensure we never use the long-form token_id
         # (PKI) as part of the cache_key.
-        unique_id = self.token_api.unique_id(token_id)
+        unique_id = self.unique_id(token_id)
         self.validate_v3_token(unique_id)
 
     @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
-                        expiration_time=CONF.token.cache_time)
+                        expiration_time=EXPIRATION_TIME)
     def _validate_token(self, token_id):
         return self.driver.validate_token(token_id)
 
     @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
-                        expiration_time=CONF.token.cache_time)
+                        expiration_time=EXPIRATION_TIME)
     def _validate_v2_token(self, token_id):
         return self.driver.validate_v2_token(token_id)
 
     @cache.on_arguments(should_cache_fn=SHOULD_CACHE,
-                        expiration_time=CONF.token.cache_time)
+                        expiration_time=EXPIRATION_TIME)
     def _validate_v3_token(self, token_id):
         return self.driver.validate_v3_token(token_id)
 
     def _is_valid_token(self, token):
-         # Verify the token has not expired.
+        """Verify the token is valid format and has not expired."""
+
         current_time = timeutils.normalize_time(timeutils.utcnow())
 
         try:
@@ -183,18 +249,17 @@ class Manager(manager.Manager):
                 expires_at = token_data['token']['expires']
             expiry = timeutils.normalize_time(
                 timeutils.parse_isotime(expires_at))
-            if current_time < expiry:
-                # Token is has not expired and has not been revoked.
-                return None
         except Exception:
             LOG.exception(_('Unexpected error or malformed token determining '
-                            'token expiry: %s') % token)
+                            'token expiry: %s'), token)
+            raise exception.TokenNotFound(_('Failed to validate token'))
 
-        # FIXME(morganfainberg): This error message needs to be updated to
-        # reflect the token couldn't be found, but this change needs to wait
-        # until Icehouse due to string freeze in Havana.  This should be:
-        # "Failed to find valid token" or something similar.
-        raise exception.TokenNotFound(_('Failed to validate token'))
+        if current_time < expiry:
+            self.check_revocation(token)
+            # Token has not expired and has not been revoked.
+            return None
+        else:
+            raise exception.TokenNotFound(_('Failed to validate token'))
 
     def _token_belongs_to(self, token, belongs_to):
         """Check if the token belongs to the right tenant.
@@ -224,9 +289,11 @@ class Manager(manager.Manager):
         self._validate_v3_token.invalidate(self, token_id)
 
 
+@six.add_metaclass(abc.ABCMeta)
 class Provider(object):
     """Interface description for a Token provider."""
 
+    @abc.abstractmethod
     def get_token_version(self, token_data):
         """Return the version of the given token data.
 
@@ -238,8 +305,9 @@ class Provider(object):
         :returns: token version string
         :raises: keystone.token.provider.UnsupportedTokenVersionException
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def issue_v2_token(self, token_ref, roles_ref=None, catalog_ref=None):
         """Issue a V2 token.
 
@@ -251,8 +319,9 @@ class Provider(object):
         :type catalog_ref: dict
         :returns: (token_id, token_data)
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def issue_v3_token(self, user_id, method_names, expires_at=None,
                        project_id=None, domain_id=None, auth_context=None,
                        metadata_ref=None, include_catalog=True):
@@ -276,8 +345,9 @@ class Provider(object):
         :type include_catalog: boolean
         :returns: (token_id, token_data)
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def revoke_token(self, token_id):
         """Revoke a given token.
 
@@ -285,8 +355,9 @@ class Provider(object):
         :type token_id: string
         :returns: None.
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def validate_token(self, token_id):
         """Detect token version and validate token and return the token data.
 
@@ -297,8 +368,9 @@ class Provider(object):
         :returns: token_data
         :raises: keystone.exception.TokenNotFound
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def validate_v2_token(self, token_id):
         """Validate the given V2 token and return the token data.
 
@@ -310,16 +382,25 @@ class Provider(object):
         :raises: keystone.exception.TokenNotFound
 
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
 
+    @abc.abstractmethod
     def validate_v3_token(self, token_id):
         """Validate the given V3 token and return the token_data.
 
         :param token_id: identity of the token
         :type token_id: string
-        :param belongs_to: project_id token belongs to
-        :type belongs_to: string
         :returns: token data
         :raises: keystone.exception.TokenNotFound
         """
-        raise exception.NotImplemented()
+        raise exception.NotImplemented()  # pragma: no cover
+
+    @abc.abstractmethod
+    def _get_token_id(self, token_data):
+        """Generate the token_id based upon the data in token_data.
+
+        :param token_data: token information
+        :type token_data: dict
+        returns: token identifier
+        """
+        raise exception.NotImplemented()  # pragma: no cover
